#!/usr/bin/env python
#-*- coding: utf-8 -*-

from PySide import QtGui, QtCore

import os
import shutil
import webbrowser
# import logging
# from datetime import datetime
from zipfile import ZipFile

from ...pinguino_core.pinguino_config import PinguinoConfig

from ..commons.repositories import Repository, ErrorModules

# from ..methods.decorators import Decorator
from ..methods.dialogs import Dialogs
# from ..methods.parser import PinguinoParser

from ..commons.widgets_features import PrettyFeatures


# Python3 compatibility
if os.getenv("PINGUINO_PYTHON") is "3":
    #Python3
    from configparser import RawConfigParser
else:
    #Python2
    from ConfigParser import RawConfigParser



########################################################################
class LibraryManager(object):
    """"""

    #----------------------------------------------------------------------
    def __init__(self):
        """"""

        if not os.path.exists(os.getenv("PINGUINO_USERLIBS_PATH")):
            os.mkdir(os.getenv("PINGUINO_USERLIBS_PATH"))

        if not os.path.exists(os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "usr")):
            os.mkdir(os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "usr"))

        if not os.path.exists(os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "library_examples")):
            os.mkdir(os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "library_examples"))

        if not os.path.exists(os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "blocks")):
            os.mkdir(os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "blocks"))

        self.connect(self.main.pushButton_libraries_install_from_zip, QtCore.SIGNAL("clicked()"), self.install_from_zip)
        self.connect(self.main.pushButton_libraries_install, QtCore.SIGNAL("clicked()"), self.install_from_repository)

        self.connect(self.main.tableWidget_user_libraries, QtCore.SIGNAL("clicked(QModelIndex)"), self.open_link)

        PrettyFeatures.LineEdit_default_text(self.main.lineEdit_libraries_source, "e.g. git@github.com:repo/pinguinolib-library.git")

        self.update_libraries_view()

        self.main.radioButton_library_git.setEnabled(ErrorModules["gitpython"])
        self.main.radioButton_library_mercurial.setEnabled(ErrorModules["hgapi"])

        self.main.pushButton_libraries_install.setEnabled(ErrorModules["gitpython"] or ErrorModules["hgapi"])
        self.main.lineEdit_libraries_source.setEnabled(ErrorModules["gitpython"] or ErrorModules["hgapi"])


    #----------------------------------------------------------------------
    def install_from_zip(self):
        """"""
        open_files = QtGui.QFileDialog.getOpenFileNames(self,
                os.getenv("PINGUINO_NAME")+QtGui.QApplication.translate("Dialogs", " - Open"),
                QtCore.QDir.home().path(),
                QtGui.QApplication.translate("Dialogs", "Zip Files (*.zip)"))

        if not open_files[0]: return
        libraries = open_files[0]

        for lib in libraries:
            name = os.path.split(lib)[1]
            name = os.path.splitext(name)[0]

            path_dirlib = os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "usr")
            # if os.path.exists(os.path.join(path_dirlib, name)):
                # Dialogs.error_message(self, QtGui.QApplication.translate("Dialogs", "Library duplicated (name conflict)."))
                # continue

            libZip = ZipFile(lib)
            libZip.extractall(path_dirlib)

            # enb = open(os.path.join(path_dirlib, name, ".enable"), "w")
            # enb.close()

            pinguino = os.path.join(path_dirlib, name, "PINGUINO")
            parser = RawConfigParser()
            parser.readfp(open(pinguino, "r"))
            info = dict(parser.items("PINGUINO"))

            new_name = "{name}".format(**info)
            self.remove_lib(new_name)

            new_dir_path = os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "usr", new_name)
            os.rename(os.path.join(path_dirlib, name), new_dir_path)

            self.post_install(new_name)

        self.update_libraries_view()


    #----------------------------------------------------------------------
    def install_from_repository(self):
        """"""
        repo_path = self.main.lineEdit_libraries_source.text()

        if repo_path.startswith("e.g."):
            Dialogs.error_message(self, "Insert a valid repository url.")
            return

        dir_path = os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "usr", "temp_lib_name")
        repo_type = self.get_repo_type()

        library_cv = Repository(repo_path, dir_path, repo_type)

        installed = library_cv.install_library()

        if not installed:
            Dialogs.error_message(self, "Impossible install from with this repository.")
            return

        pinguino = os.path.join(dir_path, "PINGUINO")
        parser = RawConfigParser()
        parser.readfp(open(pinguino, "r"))
        info = dict(parser.items("PINGUINO"))

        new_name = "{name}".format(**info)
        self.remove_lib(new_name)

        new_dir_path = os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "usr", new_name)
        os.rename(dir_path, new_dir_path)

        self.post_install(new_name)
        self.update_libraries_view()


    #----------------------------------------------------------------------
    def get_repo_type(self):
        """"""
        if self.main.radioButton_library_git.isChecked():
            return "git"
        elif self.main.radioButton_library_mercurial.isChecked():
            return "hg"


    #----------------------------------------------------------------------
    def post_install(self, lib_name):
        """"""
        lista = []

        path_examples = os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "usr", lib_name, "examples")
        dest_examples = os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "library_examples", lib_name)
        lista.append([path_examples, dest_examples])

        path_blocks = os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "usr", lib_name, "blocks")
        dest_blocks = os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "blocks", lib_name)
        lista.append([path_blocks, dest_blocks])

        for src, dest in lista:
            if os.path.exists(src):
                if os.path.exists(dest): shutil.rmtree(dest)
                shutil.copytree(src, dest)

        # self.pinguinoAPI.force_reload_libs()
        # self.update_reserved_words()
        # delattr(self, "assistant")
        self.refresh_libraries()



    #----------------------------------------------------------------------
    def get_all_libraries_paths(self):
        """"""
        paths = []
        usr = os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "usr")
        return map(lambda lib:os.path.join(usr, lib), os.listdir(usr))


    # #----------------------------------------------------------------------
    # def get_enable_libraries(self):
        # """"""
        # enable = []
        # all_libs = self.get_all_libraries_names()
        # for lib in all_libs:
            # if not os.path.exists(os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "usr", lib, ".disable")):
                # enable.append(lib)
        # return enable


    # #----------------------------------------------------------------------
    # def get_p8_libraries(self):
        # """"""
        # p8_libs = []
        # for lib in self.get_all_libraries_names():
            # p8 = os.path.exists(os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "usr", lib, "p8"))
            # if os.path.exists(p8):
                # p8_libs.append(p8)

        # return p8_libs

    # #----------------------------------------------------------------------
    # def get_p32_libraries(self):
        # """"""
        # p32_libs = []
        # for lib in self.get_all_libraries_names():
            # p32 = os.path.exists(os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "usr", lib, "p32"))
            # if os.path.exists(p32):
                # p8_libs.append(p32)

        # return p32_libs


    # #----------------------------------------------------------------------
    # def get_pdl_libraries(self):
        # """"""
        # pdl_libs = []
        # for lib in self.get_all_libraries_names():
            # pdls = os.listdir(os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "usr", lib, "pdl"))
            # for pdl in pdls:
                # if pdl.endswith(".pdl"):
                    # pdl_libs.append(os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "usr", lib, "pdl", pdl))

        # return pdl_libs


    # #----------------------------------------------------------------------
    # def get_pdl32_libraries(self):
        # """"""
        # pdl32_libs = []
        # for lib in self.get_all_libraries_names():
            # pdls = os.listdir(os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "usr", lib, "pdl"))
            # for pdl in pdls:
                # if pdl.endswith(".pdl32"):
                    # pdl32_libs.append(os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "usr", lib, "pdl", pdl))

        # return pdl32_libs


    #----------------------------------------------------------------------
    def get_library_info(self):
        """"""
        sources = []
        for lib in self.get_all_libraries_paths():
            pinguino = os.path.join(lib, "PINGUINO")
            # enable = os.path.exists(os.path.join(lib, ".disable"))
            parser = RawConfigParser()
            parser.readfp(open(pinguino, "r"))
            info = dict(parser.items("PINGUINO"))
            # info["disable"] = enable
            sources.append(info)

        return sources


    #----------------------------------------------------------------------
    def update_libraries_view(self):
        # self.ConfigLibs.save_config()
        # self.ConfigLibs.load_config()
        sources = self.get_library_info()

        rm_buttons = []
        index = 0
        self.main.tableWidget_user_libraries.setRowCount(0)

        hh = self.main.tableWidget_user_libraries.horizontalHeader()
        hh.resizeSection(0, 250)

        for lib in sources:

            name = lib["name"]
            arch = lib["arch"]
            author = lib["author"]
            description = lib["description"]
            # disable = lib["disable"]
            version = lib.get("version", "0.0")
            repository = lib["repository"]
            url = lib["url"]

            #if not installed: continue
            self.main.tableWidget_user_libraries.setRowCount(self.main.tableWidget_user_libraries.rowCount()+1)

            #name
            checkable = QtGui.QTableWidgetItem()
            # checkable.setFlags(QtCore.Qt.ItemIsEnabled|QtCore.Qt.ItemIsUserCheckable)
            # if disable: checkable.setCheckState(QtCore.Qt.Unchecked)
            # else: checkable.setCheckState(QtCore.Qt.Checked)
            checkable.setToolTip(description)
            # setattr(checkable, "library", "{}-{}".format(name, version))
            checkable.setTextAlignment(QtCore.Qt.AlignHCenter|QtCore.Qt.AlignVCenter)
            self.main.tableWidget_user_libraries.setItem(index, 0, checkable)

            #arch
            arch_ = QtGui.QTableWidgetItem()
            arch_.setFlags(QtCore.Qt.ItemIsEnabled)
            arch_.setTextAlignment(QtCore.Qt.AlignHCenter|QtCore.Qt.AlignVCenter)
            self.main.tableWidget_user_libraries.setItem(index, 1, arch_)

            #author
            auth = QtGui.QTableWidgetItem()
            auth.setFlags(QtCore.Qt.ItemIsEnabled)
            auth.setTextAlignment(QtCore.Qt.AlignHCenter|QtCore.Qt.AlignVCenter)
            self.main.tableWidget_user_libraries.setItem(index, 2, auth)

            #link
            if url:
                web_page = QtGui.QTableWidgetItem()
                web_page.setFlags(QtCore.Qt.ItemIsEnabled)
                brush = QtGui.QBrush(QtGui.QColor(0, 0, 255))
                brush.setStyle(QtCore.Qt.NoBrush)
                web_page.setForeground(brush)
                font = web_page.font()
                font.setUnderline(True)
                web_page.setFont(font)
                web_page.setTextAlignment(QtCore.Qt.AlignHCenter|QtCore.Qt.AlignVCenter)
                web_page.setToolTip(url)
                setattr(web_page, "url", url)
                self.main.tableWidget_user_libraries.setItem(index, 3, web_page)

            # #remove
            # rm = QtGui.QTableWidgetItem()
            # rm.setFlags(QtCore.Qt.ItemIsEnabled)
            # rm.setTextAlignment(QtCore.Qt.AlignHCenter|QtCore.Qt.AlignVCenter)
            # self.main.tableWidget_user_libraries.setItem(index, 4, rm)


            self.main.tableWidget_user_libraries.item(index, 0).setText("{} {}".format(name, version))
            self.main.tableWidget_user_libraries.item(index, 1).setText(arch)
            self.main.tableWidget_user_libraries.item(index, 2).setText(author)
            if url:
                self.main.tableWidget_user_libraries.item(index, 3).setText("link")

            #remove
            rm_buttons.append(QtGui.QPushButton())
            rm_buttons[-1].setText("Remove")
            self.connect(rm_buttons[-1], QtCore.SIGNAL("clicked()"), self.dummy_event(name))
            self.main.tableWidget_user_libraries.setCellWidget(index, 4, rm_buttons[-1])

            index += 1


    #----------------------------------------------------------------------
    def dummy_event(self, lib):
        """"""
        def button():
            self.remove_lib(lib)
        return button


    #----------------------------------------------------------------------
    def open_link(self, model_index):
        """"""

        # column = model_index.column()
        # if column == 0:
            # self.update_eneables()

        item = self.main.tableWidget_user_libraries.itemFromIndex(model_index)
        url = getattr(item, "url", None)
        if not url: return

        reply = Dialogs.confirm_message(self, QtGui.QApplication.translate("Dialogs", "Do you want open this URL in a new tab?")+"\n"+url)
        if reply: webbrowser.open_new_tab(url)


    # #----------------------------------------------------------------------
    # def update_eneables(self):
        # """"""
        # for index in range(self.main.tableWidget_user_libraries.rowCount()):
            # item = self.main.tableWidget_user_libraries.item(index, 0)
            # if item.checkState() == QtCore.Qt.CheckState.Checked:
                # self.enable_lib(item.library)
            # else:
                # self.disable_lib(item.library)


    # #----------------------------------------------------------------------
    # def enable_lib(self, library):
        # """"""
        # path_enable = os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "usr", library, ".disable")
        # if os.path.exists(path_enable):
            # os.remove(path_enable)


    # #----------------------------------------------------------------------
    # def disable_lib(self, library):
        # """"""
        # path_enable = os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "usr", library, ".disable")
        # if not os.path.exists(path_enable):
            # enb = open(path_enable, "w")
            # enb.close()

    #----------------------------------------------------------------------
    def remove_lib(self, library):
        """"""
        if library in os.listdir(os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "usr")):
            reply = Dialogs.confirm_message(self, QtGui.QApplication.translate("Dialogs", "Do you want remove this library?")+"\n"+library)
            for dir_ in ["usr", "library_examples", "blocks"]:
                path = os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), dir_, library)
                if os.path.exists(path): shutil.rmtree(path)

            self.update_libraries_view()
            self.refresh_libraries()


    #----------------------------------------------------------------------
    def refresh_libraries(self):
        """"""
        self.pinguinoAPI.force_reload_libs()
        PinguinoConfig.update_user_libs(self.pinguinoAPI)
        self.update_reserved_words()
        if hasattr(self, "assistant"): delattr(self, "assistant")
        self.change_dir_files(0)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/yeison/Documentos/Desarrollo/Pinguino/GitHub/pinguino-ide/pinguino/qtgui/frames/select_area.ui'
#
# Created: Tue Nov 17 17:51:45 2015
#      by: pyside-uic 0.2.15 running on PySide 1.2.4
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_Selection(object):
    def setupUi(self, Selection):
        Selection.setObjectName("Selection")
        Selection.resize(360, 145)
        Selection.setWindowTitle("SelectArea")
        self.gridLayout = QtGui.QGridLayout(Selection)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setSpacing(0)
        self.gridLayout.setObjectName("gridLayout")
        self.border_w = QtGui.QFrame(Selection)
        self.border_w.setMinimumSize(QtCore.QSize(1, 0))
        self.border_w.setMaximumSize(QtCore.QSize(1, 16777215))
        self.border_w.setStyleSheet("background-image: url(:/selection/selection/border.png);")
        self.border_w.setObjectName("border_w")
        self.horizontalLayout_7 = QtGui.QHBoxLayout(self.border_w)
        self.horizontalLayout_7.setSpacing(0)
        self.horizontalLayout_7.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.gridLayout.addWidget(self.border_w, 1, 0, 1, 1)
        self.background = QtGui.QFrame(Selection)
        self.background.setStyleSheet("background-image: url(:/selection/selection/background.png);")
        self.background.setObjectName("background")
        self.horizontalLayout_2 = QtGui.QHBoxLayout(self.background)
        self.horizontalLayout_2.setSpacing(0)
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.gridLayout.addWidget(self.background, 1, 1, 1, 1)
        self.border_e = QtGui.QFrame(Selection)
        self.border_e.setMinimumSize(QtCore.QSize(1, 0))
        self.border_e.setMaximumSize(QtCore.QSize(1, 16777215))
        self.border_e.setStyleSheet("background-image: url(:/selection/selection/border.png);")
        self.border_e.setObjectName("border_e")
        self.horizontalLayout_9 = QtGui.QHBoxLayout(self.border_e)
        self.horizontalLayout_9.setSpacing(0)
        self.horizontalLayout_9.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.gridLayout.addWidget(self.border_e, 1, 2, 1, 1)
        self.border_n = QtGui.QFrame(Selection)
        self.border_n.setMinimumSize(QtCore.QSize(0, 1))
        self.border_n.setMaximumSize(QtCore.QSize(16777215, 1))
        self.border_n.setStyleSheet("background-image: url(:/selection/selection/border.png);")
        self.border_n.setObjectName("border_n")
        self.horizontalLayout_11 = QtGui.QHBoxLayout(self.border_n)
        self.horizontalLayout_11.setSpacing(0)
        self.horizontalLayout_11.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.gridLayout.addWidget(self.border_n, 0, 0, 1, 3)
        self.border_s = QtGui.QFrame(Selection)
        self.border_s.setMinimumSize(QtCore.QSize(0, 1))
        self.border_s.setMaximumSize(QtCore.QSize(16777215, 1))
        self.border_s.setStyleSheet("background-image: url(:/selection/selection/border.png);")
        self.border_s.setObjectName("border_s")
        self.horizontalLayout_5 = QtGui.QHBoxLayout(self.border_s)
        self.horizontalLayout_5.setSpacing(0)
        self.horizontalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.gridLayout.addWidget(self.border_s, 2, 0, 1, 3)

        self.retranslateUi(Selection)
        QtCore.QMetaObject.connectSlotsByName(Selection)

    def retranslateUi(self, Selection):
        pass

import resources_rc



#!/usr/bin/env python
#-*- coding: utf-8 -*-

#----------------------------------------------------------------------
def setTab(name):

    global TAB
    TAB = name


#----------------------------------------------------------------------
def setDic(dic):

    global set_
    set_ = dic


#----------------------------------------------------------------------
def addOper2(name):

    set_[name] = ["output", TAB]
    set_[name].append(["syntax", "("])
    set_[name].append(["space"])
    set_[name].append(["label", name])
    set_[name].append(["space"])
    set_[name].append(["syntax", ")"])


#----------------------------------------------------------------------
def addOper1(name, space="space"):

    set_[name] = ["output", TAB]
    set_[name].append(["label", name])
    set_[name].append(["syntax", "("])
    set_[name].append([space])
    set_[name].append(["syntax", ")"])


#----------------------------------------------------------------------
def addOper2_bool(name, space="space"):

    set_["".join(name)] = ["output-bool", TAB]
    set_["".join(name)].append(["syntax", "("])
    set_["".join(name)].append([space])
    if type(name) == type([]):
        set_["".join(name)].append(["syntax", name[0]])
        set_["".join(name)].append(["label", name[1]])
        set_["".join(name)].append(["syntax", name[2]])
    else: set_["".join(name)].append(["label", name])

    set_["".join(name)].append([space])
    set_["".join(name)].append(["syntax", ")"])


#----------------------------------------------------------------------
def addOper1_bool(name, space="space"):

    set_["".join(name)] = ["output-bool", TAB]
    if type(name) == type([]):
        set_["".join(name)].append(["syntax", name[0]])
        set_["".join(name)].append(["label", name[1]])
        set_["".join(name)].append(["syntax", name[2]])
    else: set_["".join(name)].append(["label", name])
    set_["".join(name)].append(["syntax", "("])
    set_["".join(name)].append([space])
    set_["".join(name)].append(["syntax", ")"])


#----------------------------------------------------------------------
def addNewFunct(name, nvars, tipo="linear", bools=[], full=[]):

    set_[name] = [tipo, TAB]
    set_[name].append(["label", name])
    set_[name].append(["syntax", "("])

    for i in range(nvars-1):
        if i in bools: set_[name].append(["space-bool"])
        else: set_[name].append(["space"])
        set_[name].append(["syntax", ","])
    if nvars > 0: set_[name].append(["space"])
    set_[name].append(["syntax", ")"])
    set_[name].append(["full", full])


#----------------------------------------------------------------------
def addNewSentence(name, space=False, tipo="linear", extra=""):

    set_[name+extra] = [tipo, TAB]
    set_[name+extra].append(["label", name])
    set_[name+extra].append(["syntax", " "])
    if space: set_[name+extra].append(["space"])



########################################################################
class Kit(object):

    #----------------------------------------------------------------------
    def addBackSpace(self):
        self.bloque.append(["back_space"])


    #----------------------------------------------------------------------
    def addLabel(self, str_label):
        self.bloque.append(["label", str_label])


    #----------------------------------------------------------------------
    def addSpace(self):
        self.bloque.append(["space"])


    #----------------------------------------------------------------------
    def addSpaceBool(self):
        self.bloque.append(["space_bool"])


    #----------------------------------------------------------------------
    def addSyntax(self, str_syntax):
        self.bloque.append(["syntax", str_syntax])


    #----------------------------------------------------------------------
    def addButton(self, str_label, checkeable=False, checked=False):
        self.bloque.append(["button", str_label, checkeable, checked])


    #----------------------------------------------------------------------
    def addSpinChoice(self, str_default, list_choices):
        self.bloque.append(["spin_choice", str_default, list_choices])


    #----------------------------------------------------------------------
    def addEdit(self, str_default, background="white", foreground=(90, 90, 90)):
        self.bloque.append(["edit", str_default, background, foreground])


    #----------------------------------------------------------------------
    def addSpinInt(self, start=0, min_=-100, max_=100):
        self.bloque.append(["spin_int", start, min_, max_])


    #----------------------------------------------------------------------
    def addSpinFloat(self, start=0.1, step=0.1, min_=-100, max_=100):
        self.bloque.append(["spin_float", start, step, min_, max_])


    #----------------------------------------------------------------------
    def addSlider(self, start=0, min_=0, max_=1023):
        self.bloque.append(["slider", start, min_, max_])


    #----------------------------------------------------------------------
    def addDecorator(self, str_label):
        self.bloque.append(["decorator", str_label])


    #----------------------------------------------------------------------
    def addArg(self, arg, space=True):

        self.addDecorator("%s="%arg)
        if space: self.addSpace()
        else: self.addSpaceBool()


    #----------------------------------------------------------------------
    def addContextHelp(self, str_help):
        self.bloque.append(["help", str_help])


    #----------------------------------------------------------------------
    def addCode(self, code_pinguino, code_python=""):
        self.bloque.append(["code", {"pinguino": code_pinguino, "python": code_python}])


    #----------------------------------------------------------------------
    def fillWith(self, list_name_blocks):
        self.bloque.append(["full", list_name_blocks])


    #----------------------------------------------------------------------
    def getBlock(self):
        return self.bloque


    #----------------------------------------------------------------------
    def getType(self, t, tab):

        if t == "linear":return(Linear(tab))
        elif t == "output":return(Output(tab))
        elif t == "output-bool":return(OutputBool(tab))
        elif t == "function":return(Function(tab))


    #----------------------------------------------------------------------
    def addFunct(self, tab, name, nvars, t="linear", fill=[], str_help=None):

        pm = self.getType(t, tab)
        pm.addLabel(name)
        pm.addSyntax("(")
        for i in range(nvars):
            pm.addSpace()
            if not i == nvars - 1: pm.addSyntax(",")
        pm.addSyntax(")")
        if fill: pm.fillWith(fill)
        if str_help: pm.addContextHelp(str_help)
        return pm.getBlock()


    #----------------------------------------------------------------------
    def addOper(self, tab, label, form="output", spaces=[], help=None):

        pm = self.getType(form, tab)
        trad = False
        if label.startswith("@") and label.endswith("@"):
            trad = True
            label = label[1:-1]
        if len(spaces) == 1:
            if trad: pm.addSyntax("@")
            pm.addLabel(label)
            if trad: pm.addSyntax("@")
            pm.addSyntax("(")
            if spaces[0] == "space": pm.addSpace()
            elif spaces[0] == "space_bool": pm.addSpaceBool()
            pm.addSyntax(")")
        elif len(spaces) == 2:
            pm.addSyntax("(")
            if spaces[0] == "space": pm.addSpace()
            elif spaces[0] == "space_bool": pm.addSpaceBool()
            if trad: pm.addSyntax("@")
            pm.addLabel(label)
            if trad: pm.addSyntax("@")
            if spaces[1] == "space": pm.addSpace()
            elif spaces[1] == "space_bool": pm.addSpaceBool()
            pm.addSyntax(")")
        if help: pm.addContextHelp(help)
        return pm.getBlock()


class Linear(Kit):
    def __init__(self, tab): self.bloque = ["linear", tab]

class Output(Kit):
    def __init__(self, tab): self.bloque = ["output", tab]

class OutputBool(Kit):
    def __init__(self, tab): self.bloque = ["output-bool", tab]

class Function(Kit):
    def __init__(self, tab): self.bloque = ["function", tab]

class Nested(Kit):
    def __init__(self, tab): self.bloque = ["nested", tab]

class NestedNext(Kit):
    def __init__(self, tab): self.bloque = ["nested-second", tab]

class FrameText(Kit):
    def __init__(self, tab): self.bloque = ["frame-edit", tab]

class FramePlot(Kit):
    def __init__(self, tab): self.bloque = ["frame-plot", tab]

class FrameDebug(Kit):
    def __init__(self, tab): self.bloque = ["frame-debug", tab]





#!/usr/bin/env python
#-*- coding: utf-8 -*-

import os
import logging
import shutil
import platform
import sys

# Python3 compatibility
if os.getenv("PINGUINO_PYTHON") is "3":
    #Python3
    from configparser import RawConfigParser
else:
    #Python2
    from ConfigParser import RawConfigParser

########################################################################
class PinguinoConfig(object):

    #----------------------------------------------------------------------
    @classmethod
    def set_environ_vars(cls):
        """Configure some environ variables about OS and architecture.
        """

        if not os.path.exists(os.path.join(os.getenv("PINGUINO_LIB"), "paths.cfg")):
            logging.error("Missing: "+os.path.join(os.getenv("PINGUINO_LIB"), "paths.cfg"))
            sys.exit()

        config_paths = RawConfigParser()
        config_paths.readfp(open(os.path.join(os.getenv("PINGUINO_LIB"), "paths.cfg"), "r"))

        #RB20141116 : get the “bitness” of the current OS
        bitness, linkage = platform.architecture()
        os.environ["PINGUINO_OS_ARCH"] = bitness

        if os.name == "posix": #GNU/Linux
            os.environ["PINGUINO_OS_NAME"] = "linux"

        #Mac could return posix :/
        elif os.name == "os2":  #Mac OS X
            os.environ["PINGUINO_OS_NAME"] = "macosx"

        elif os.name == "nt":  #Windows
            os.environ["PINGUINO_OS_NAME"] = "windows"

        #load path from paths.conf
        os.environ["PINGUINO_USER_PATH"] = os.path.expandvars(os.path.expanduser(config_paths.get("paths-%s"%os.getenv("PINGUINO_OS_NAME"), "user_path")))
        os.environ["PINGUINO_INSTALL_PATH"] = os.path.expandvars(os.path.expanduser(config_paths.get("paths-%s"%os.getenv("PINGUINO_OS_NAME"), "install_path")))
        os.environ["PINGUINO_USERLIBS_PATH"] = os.path.expandvars(os.path.join(os.getenv("PINGUINO_USER_PATH"), "libraries"))


    #----------------------------------------------------------------------
    @classmethod
    def check_user_files(cls):
        """Check existence of depends directories and created if missed."""

        #check instalation path
        if not os.path.exists(os.getenv("PINGUINO_INSTALL_PATH")):
            logging.warning("Missing Pinguino libraries instalation. "
                            "Make sure the paths in %s are correct."%os.path.join(os.getenv("PINGUINO_LIB"), "paths.conf"))

        #create ~/.pinguino
        if not os.path.exists(os.getenv("PINGUINO_USER_PATH")):
            os.mkdir(os.getenv("PINGUINO_USER_PATH"))

        #Check files and directories
        cls.check_locations()


    #----------------------------------------------------------------------
    @classmethod
    def check_locations(cls):
        """If not exist some directory then copy them from installation path."""

        cls.if_not_exist_then_copy(src=os.path.join(os.getenv("PINGUINO_INSTALL_PATH"), "examples"),
                                   dst=os.path.join(os.getenv("PINGUINO_USER_PATH"), "examples"))

        cls.if_not_exist_then_copy(src=os.path.join(os.getenv("PINGUINO_INSTALL_PATH"), "graphical_examples"),
                                   dst=os.path.join(os.getenv("PINGUINO_USER_PATH"), "graphical_examples"))

        cls.if_not_exist_then_copy(src=os.path.join(os.getenv("PINGUINO_INSTALL_PATH"), "source"),
                                   dst=os.path.join(os.getenv("PINGUINO_USER_PATH"), "source"))

        cls.if_not_exist_then_copy(src=os.path.join(os.getenv("PINGUINO_INSTALL_PATH"), "p32", "obj"),
                                   dst=os.path.join(os.getenv("PINGUINO_USER_PATH"), "source", "obj"))

        cls.if_not_exist_then_copy(src=os.path.join(os.getenv("PINGUINO_LIB"), "qtgui", "config", "pinguino.%s.conf"%os.getenv("PINGUINO_OS_NAME")),
                                   dst=os.path.join(os.getenv("PINGUINO_USER_PATH"), "pinguino.conf"))


    #----------------------------------------------------------------------
    @classmethod
    def if_not_exist_then_copy(cls, src, dst):
        """The method name say all.

        Parameters
        ----------
        src: str
            Source file or directory.

        dst: str
            Destiny file or directory.
        """

        #When src is a file
        if os.path.isfile(src):
            if not os.path.exists(src): logging.warning("Missing: " + src)
            else:
                if not os.path.exists(dst): shutil.copy(src, dst)
            return

        #Create parent directories
        def prepare_directory(file_path):
            dir_, filename = os.path.split(file_path)
            all_dirs = []
            while not os.path.exists(dir_):
                all_dirs.append(dir_)
                dir_, dir2 = os.path.split(dir_)
            all_dirs.reverse()
            for dir_ in all_dirs:  os.mkdir(dir_)

        #When src is a directory, copy file by file
        all_files = []
        for path, dirs, files in os.walk(src):
            for f in files:
                file_path_src = os.path.join(path, f)
                file_path_dst = dst + os.path.join(path, f).replace(src, "")

                if not os.path.exists(file_path_dst):
                    logging.warning("Copying: " + file_path_dst)
                    prepare_directory(file_path_dst)
                    shutil.copy(file_path_src, file_path_dst)


    #----------------------------------------------------------------------
    @classmethod
    def update_pinguino_extra_options(self, config, pinguino_object):
        """Load on Pinguino Object the advance configuration for 32 bit.

        Parameters
        ----------
        config: RawConfigParser
            Config file.
        pinguino_object: Pinguino()
            Pinguino
        """

        pinguino_object.MIPS16 = str(config.config("Board", "mips16", True)).lower()
        pinguino_object.HEAPSIZE = str(config.config("Board", "heapsize", 512))
        pinguino_object.OPTIMIZATION = config.config("Board", "optimization", "-O3")


    #----------------------------------------------------------------------
    @classmethod
    def update_pinguino_paths(cls, config, pinguino_object, prefix=None, workspace=None):
        """Load paths of libraries and compilers.

        Parameters
        ----------
        config: RawConfigParser
            Config file.
        pinguino_object: Pinguino()
            Pinguino
        prefix: str, optional
            Path for search libraries and compilers, default is absolute path from root.
        workspace: str,optional
            Path for makefiles and temp files, default is ~/.pinguino/source

        """

        user_sdcc_bin = config.get_path("sdcc_bin", prefix)
        if user_sdcc_bin: pinguino_object.P8_SDCC_BIN = user_sdcc_bin
        else: logging.warning("Missing path to 'sdcc_bin': '%s'"%user_sdcc_bin)

        user_sdcc_bin = config.get_path("xc8_bin", prefix)
        if user_sdcc_bin: pinguino_object.P8_XC8_BIN = user_sdcc_bin
        else: logging.warning("Missing path to 'xc8_bin': '%s'"%user_sdcc_bin)

        user_gcc_bin = config.get_path("gcc_bin", prefix)
        if user_gcc_bin: pinguino_object.P32_GCC_BIN = user_gcc_bin
        else: logging.warning("Missing path to 'gcc_bin': '%s'"%user_gcc_bin)

        if workspace:
            pinguino_object.SOURCE_DIR = workspace
        else:
            pinguino_source = os.path.join(os.getenv("PINGUINO_USER_PATH"), "source")
            if pinguino_source: pinguino_object.SOURCE_DIR = pinguino_source
            else: logging.warning("Missing path to 'source': '%s'"%pinguino_source)

        pinguino_8_libs = config.get_path("pinguino_8_libs", prefix)
        if pinguino_8_libs: pinguino_object.P8_DIR = pinguino_8_libs
        else: logging.warning("Missing path to '8_libs': '%s'"%pinguino_8_libs)

        pinguino_32_libs = config.get_path("pinguino_32_libs", prefix)
        if pinguino_32_libs: pinguino_object.P32_DIR = pinguino_32_libs
        else: logging.warning("Missing path to '32_libs': '%s'"%pinguino_32_libs)


    #----------------------------------------------------------------------
    @classmethod
    def update_user_libs(cls, pinguino_object):
        """Reload paths for user installed libraries.

        Parameters
        ----------
        pinguino_object: Pinguino()
            Pinguino
        """

        pinguino_object.USER_P8_LIBS = cls.get_p8_libraries()
        pinguino_object.USER_P32_LIBS = cls.get_p32_libraries()
        pinguino_object.USER_PDL = cls.get_pdl_libraries()


    #----------------------------------------------------------------------
    @classmethod
    def get_p8_libraries(cls):
        """
        This is path were is .c files.

        """
        p8_libs = []
        if os.path.exists(os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "usr")):
            for lib in os.listdir(os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "usr")):
                p8 = os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "usr", lib, "p8")
                if os.path.exists(p8):
                    p8_libs.append(p8)

        os_name = os.getenv("PINGUINO_OS_NAME")
        if os.getenv("PINGUINO_P8", False):
            if os_name == "windows":
                p8_libs.extend(os.getenv("PINGUINO_P8").split(";"))
            elif os_name == "linux":
                p8_libs.extend(os.getenv("PINGUINO_P8").split(":"))
            #FIXME_OSX
            # elif os_name == "macosx":
                # p32_libs.extend(os.getenv(PINGUINO_P32).aplit(":"))


        return p8_libs


    #----------------------------------------------------------------------
    @classmethod
    def get_p32_libraries(cls):
        """

        This is path were is .c files.

        """
        p32_libs = []
        if os.path.exists(os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "usr")):
            for lib in os.listdir(os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "usr")):
                p32 = os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "usr", lib, "p32")
                if os.path.exists(p32):
                    p32_libs.append(p32)

        os_name = os.getenv("PINGUINO_OS_NAME")
        if os.getenv("PINGUINO_P32", False):
            if os_name == "windows":
                p32_libs.extend(os.getenv("PINGUINO_P32").split(";"))
            elif os_name == "linux":
                p32_libs.extend(os.getenv("PINGUINO_P32").split(":"))
            #FIXME_OSX
            # elif os_name == "macosx":
                # p32_libs.extend(os.getenv(PINGUINO_P32).aplit(":"))

        return p32_libs


    #----------------------------------------------------------------------
    @classmethod
    def get_pdl_libraries(cls):
        """

        This is individual paths for each .pdl or pdl32.

        """
        pdl_libs = []
        if os.path.exists(os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "usr")):
            for lib in os.listdir(os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "usr")):
                pdls = os.listdir(os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "usr", lib, "pdl"))
                for pdl in pdls:
                    if pdl.endswith(".pdl") or pdl.endswith(".pdl32"):
                        pdl_libs.append(os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "usr", lib, "pdl", pdl))

        os_name = os.getenv("PINGUINO_OS_NAME")
        if os.getenv("PINGUINO_PDL", False):
            if os_name == "windows":
                pdl_libs.extend(os.getenv("PINGUINO_PDL").split(";"))
            elif os_name == "linux":
                pdl_libs.extend(os.getenv("PINGUINO_PDL").split(":"))
            #FIXME_OSX
            # elif os_name == "macosx":
                # pdl_libs.extend(os.getenv(PINGUINO_P32).aplit(":"))


        return pdl_libs



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/yeison/Documentos/Desarrollo/Pinguino/GitHub/pinguino-ide/pinguino/qtgui/frames/insert_block.ui'
#
# Created: Tue Nov 17 17:51:45 2015
#      by: pyside-uic 0.2.15 running on PySide 1.2.4
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_InsertBlock(object):
    def setupUi(self, InsertBlock):
        InsertBlock.setObjectName("InsertBlock")
        InsertBlock.resize(400, 107)
        self.gridLayout = QtGui.QGridLayout(InsertBlock)
        self.gridLayout.setContentsMargins(5, 5, 5, 5)
        self.gridLayout.setHorizontalSpacing(0)
        self.gridLayout.setVerticalSpacing(2)
        self.gridLayout.setObjectName("gridLayout")
        self.listWidget = QtGui.QListWidget(InsertBlock)
        self.listWidget.setObjectName("listWidget")
        self.gridLayout.addWidget(self.listWidget, 1, 0, 1, 2)
        self.lineEdit = QtGui.QLineEdit(InsertBlock)
        self.lineEdit.setObjectName("lineEdit")
        self.gridLayout.addWidget(self.lineEdit, 0, 0, 1, 2)

        self.retranslateUi(InsertBlock)
        QtCore.QMetaObject.connectSlotsByName(InsertBlock)
        InsertBlock.setTabOrder(self.lineEdit, self.listWidget)

    def retranslateUi(self, InsertBlock):
        InsertBlock.setWindowTitle(QtGui.QApplication.translate("InsertBlock", "Insert Block", None, QtGui.QApplication.UnicodeUTF8))




#!/usr/bin/env python
#-*- coding: utf-8 -*-

import inspect
import os
import sys
#import pickle

allBlocks = {}

########################################################################
class UserBlocks(object):

    #----------------------------------------------------------------------
    def __init__(self):

        path = os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "blocks")
        if not os.path.exists(path): return
        sets = os.listdir(os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), "blocks"))
        sys.path.append(os.path.join(os.getenv("PINGUINO_LIB"), "qtgui", "gide", "py_bloques"))

        for dir_ in sets:
            files = os.listdir(os.path.join(path, dir_))
            files = filter(lambda f:f.endswith("py") and f != "__init__.py" and (not f.startswith(".")), files)
            sys.path.append(os.path.join(path, dir_))
            for module in files:
                mod = __import__(module.replace(".py", ""))
                self.extractBlocks(getattr(mod, "PinguinoBlock"))
            sys.path.remove(os.path.join(path, dir_))
        sys.path.remove(os.path.join(os.getenv("PINGUINO_LIB"), "qtgui", "gide", "py_bloques"))


    #----------------------------------------------------------------------
    def getBlocks(self):

        return allBlocks


    #----------------------------------------------------------------------
    def extractBlocks(self, mod):

        alls = inspect.getmembers(mod())
        for block in alls:
            if hasattr(block[1], "im_class"):
                if block[0] != "__init__" and block[1].im_class == mod().__init__.im_class:
                    allBlocks[block[0]] = block[1]()



#!/usr/bin/env python
#-*- coding: utf-8 -*-

import os
import sys
import pickle

from PySide import QtCore, QtGui

from .python_highlighter import Highlighter
from .python_shell import PythonShell

HEAD = os.getenv("PINGUINO_NAME") + " " + os.getenv("PINGUINO_VERSION") + "\n" + "Python " + sys.version + " on " + sys.platform
HELP = QtGui.QApplication.translate("PythonShell", "Commands available:") + ' "clear", "restart"'

START = ">>> "
NEW_LINE = "... "


########################################################################
class PinguinoTextEdit(QtGui.QPlainTextEdit):

    #----------------------------------------------------------------------
    def __init__(self, widget=None, shell=True):
        super(PinguinoTextEdit, self).__init__(widget)

        self.shell = PythonShell()

        self.setLineWrapMode(self.NoWrap)

        if shell:
            self.appendPlainText(HEAD)
            self.appendPlainText(HELP)
            self.appendPlainText(START)

            self.extra_args = {}

            self.load_historial()
            self.index_historial = 0

            self.multiline_commands = []

        Highlighter(self.document(), extra=[START, NEW_LINE], python=shell)
        # self.connect(self, QtCore.SIGNAL("textChanged(QString)"), self.textChanged)
        self.setFrameShape(QtGui.QFrame.NoFrame)


    #----------------------------------------------------------------------
    def update_historial(self, element):
        """"""
        filename = os.path.join(os.getenv("PINGUINO_USER_PATH"), ".shellhistory")
        if os.path.exists(filename):
            self.load_historial()
            self.historial.append(element)
            self.historial = self.historial[-100:]
        else:
            self.historial = [element]

        pickle.dump(self.historial, open(filename, "wb"), protocol=2)


    #----------------------------------------------------------------------
    def load_historial(self):
        """"""
        filename = os.path.join(os.getenv("PINGUINO_USER_PATH"), ".shellhistory")
        if os.path.exists(filename):
            self.historial = pickle.load(open(filename, "rb"))
        else:
            pickle.dump([], open(filename, "wb"), protocol=2)
            self.historial = []



    #----------------------------------------------------------------------
    def write(self, text, prefix=""):
        # if not self.checkbox_debug.isChecked(): return
        # return self.log_output(text[:-1])

        if prefix:
            text = text.replace("\n", "\n{} ".format(prefix))
            text = text.replace("\\n", "\\n{} ".format(prefix))
            text = "{} {}".format(prefix, text)
        else:
            text = text.replace("\n", "\n")
            text = text.replace("\\n", "\\n")


        if not (text.endswith("\n") or text.endswith("\n ")):
            text += "\n"

        # for i in range(100): print(i)

        self.moveCursor(QtGui.QTextCursor.EndOfLine)
        self.moveCursor(QtGui.QTextCursor.End)
        self.insertPlainText(text)
        self.moveCursor(QtGui.QTextCursor.End)

        self.repaint()


    #----------------------------------------------------------------------
    def keyPressEvent(self, event):

        self.set_last_line()

        if event.key() in (QtCore.Qt.Key_Enter, QtCore.Qt.Key_Enter-1):
            self.moveCursor(QtGui.QTextCursor.End)
            self.index_historial = 0
            super(PinguinoTextEdit, self).keyPressEvent(event)

            command = self.get_command()

            if self.multiline_commands:
                self.multiline_commands.append(command)
                #return

                if command.endswith("\n... \n"):
                    #mcommand = ";".join(self.multiline_commands)
                    command = command.replace("\n", ";")
                    command = command.replace(":;", ":").replace(":;", ":").replace(":;", ":")
                    command = command.replace(NEW_LINE, "")
                    command = command[:-2]
                    command += "\n"
                    if self.run_default_command(command):
                        self.appendPlainText(START)
                        self.multiline_commands = []
                        return
                    else:
                        self.multiline_commands = []
                        pass

                else:
                    self.insertPlainText(NEW_LINE)
                    return

            elif command.endswith(":\n"):
                self.multiline_commands.append(command)
                self.insertPlainText(NEW_LINE)
                return

            elif self.run_default_command(command):
                self.appendPlainText(START)
                return

            self.update_historial(command.replace("\n", ""))

            if not command.isspace():
                self.moveCursor(QtGui.QTextCursor.End)
                # self.insertPlainText(self.shell.run(command))
                self.shell.run(command, self)
            self.insertPlainText(START)
            self.moveCursor(QtGui.QTextCursor.End)

        elif event.key() == QtCore.Qt.Key_Backspace:
            if not self.get_command(): return
            else: super(PinguinoTextEdit, self).keyPressEvent(event)

        elif event.key() == QtCore.Qt.Key_Up:
            if len(self.historial) >= self.index_historial + 1:
                self.index_historial += 1
                self.moveCursor(QtGui.QTextCursor.End)

                tc = self.textCursor()
                tc.movePosition(tc.StartOfLine, tc.KeepAnchor)
                tc.removeSelectedText()
                tc.insertText(START)

                tc.insertText(self.historial[-self.index_historial])

        elif event.key() == QtCore.Qt.Key_Down:
            if len(self.historial) >= self.index_historial - 1:
                self.index_historial -= 1
                if self.index_historial <= 0:
                    self.index_historial += 1
                    return
                self.moveCursor(QtGui.QTextCursor.End)

                tc = self.textCursor()
                tc.movePosition(tc.StartOfLine, tc.KeepAnchor)
                tc.removeSelectedText()
                tc.insertText(START)

                tc.insertText(self.historial[-self.index_historial])

        elif event.key() == QtCore.Qt.Key_Left:
            if self.no_overwrite_start():
                super(PinguinoTextEdit, self).keyPressEvent(event)

        elif event.key() == QtCore.Qt.Key_Tab:
            tc = self.textCursor()
            tc.movePosition(tc.WordLeft, tc.KeepAnchor)
            word = tc.selectedText()
            object_ = None
            attr_ = ""

            tc.movePosition(tc.WordLeft, tc.KeepAnchor)
            n_word = tc.selectedText()

            if n_word == ".":
                tc.movePosition(tc.WordLeft, tc.KeepAnchor)
                object_ = tc.selectedText()

            if n_word.startswith("."):
                tc.movePosition(tc.WordLeft, tc.KeepAnchor)
                tc.movePosition(tc.WordLeft, tc.KeepAnchor)
                if tc.selectedText().startswith("."):
                    while tc.selectedText().startswith("."):
                        tc.movePosition(tc.WordLeft, tc.KeepAnchor)
                else:
                    tc.movePosition(tc.WordRight, tc.KeepAnchor)

                object_, attr_ = tc.selectedText().rsplit(".", 1)


            if n_word.endswith("."):
                tc.movePosition(tc.WordLeft, tc.KeepAnchor)
                if tc.selectedText().startswith("."):
                    while tc.selectedText().startswith("."):
                        tc.movePosition(tc.WordLeft, tc.KeepAnchor)
                else:
                    tc.movePosition(tc.WordRight, tc.KeepAnchor)

                object_, attr_ = tc.selectedText().rsplit(".", 1)

            if object_:
                objects_ = object_.split(".")
                s = self.shell.statement_module
                for ob in objects_:
                    s = getattr(s, ob)

                options = filter(lambda x:x.startswith(attr_), s.__dict__.keys())

            elif word:
                s = self.shell.statement_module
                options = filter(lambda x:x.startswith(word), s.__dict__.keys())

            options = list(filter(lambda x:not x.startswith("__"), options))

            common = os.path.commonprefix(options)
            if common and word != common: options = [common]

            if len(options) > 1:
                self.moveCursor(tc.StartOfLine)
                i = 0
                options = sorted(options)
                for option in options:
                    i += 1
                    if i == 3:
                        i = 0
                        self.insertPlainText("{}\n".format(option))
                    else: self.insertPlainText("{}".format(option).ljust(50," "))
                self.insertPlainText("\n\n")
                self.moveCursor(tc.End)

            elif len(options) == 1:
                self.moveCursor(tc.End)

                if object_:
                    tc.movePosition(tc.WordRight, tc.KeepAnchor)
                    while "." in tc.selectedText():
                        tc.movePosition(tc.WordRight, tc.KeepAnchor)
                    tc.removeSelectedText()

                elif word:
                    tc.movePosition(tc.WordRight, tc.KeepAnchor)
                    tc.removeSelectedText()

                self.insertPlainText(options[0])
                self.moveCursor(tc.End)

        elif event.key() == QtCore.Qt.Key_Home:
            super(PinguinoTextEdit, self).keyPressEvent(event)
            tc = self.textCursor()
            self.moveCursor(tc.WordRight, tc.MoveAnchor)

        elif event.key() in [QtCore.Qt.Key_PageUp, QtCore.Qt.Key_PageDown]:
            pass

        else:
            super(PinguinoTextEdit, self).keyPressEvent(event)


    #----------------------------------------------------------------------
    def get_command(self):
        plain = self.toPlainText()
        comand = plain[plain.rfind(START):]
        return comand[len(START):]


    #----------------------------------------------------------------------
    def wheelEvent(self, event):
        if event.modifiers() == QtCore.Qt.ControlModifier:
            self.step_font_size(event.delta())

        else: super(PinguinoTextEdit, self).wheelEvent(event)


    ##----------------------------------------------------------------------
    #def (self):
        #""""""


    #----------------------------------------------------------------------
    def contextMenuEvent(self, event):
        menu = QtGui.QMenu()

        # if hasattr(self, "historial"):
            # if self.historial:
                # sub_menu = QtGui.QMenu(QtGui.QApplication.translate("PythonShell", "Last commands"))
                # rhistorial = self.historial[:]

                # rhistorial.reverse()

                # rhistorial = filter(lambda cm: not cm.isspace(), rhistorial)
                # rhistorial = filter(lambda cm: not cm == "", rhistorial)

                # for command in rhistorial[:10]:
                    # sub_menu.addAction(command, lambda :self.insertPlainText(command))
                # menu.addMenu(sub_menu)
                # menu.addSeparator()

        if self.isReadOnly():
            menu.addAction(QtGui.QApplication.translate("PythonShell", "Clear"), self.command_clear)
            # menu.addAction(QtGui.QApplication.translate("PythonShell", "Restart"), self.command_restart)
            # menu.addSeparator()
            # menu.addAction(QtGui.QApplication.translate("PythonShell", "Cut"), self.cut, QtGui.QKeySequence.Cut)
            menu.addAction(QtGui.QApplication.translate("PythonShell", "Copy"), self.copy, QtGui.QKeySequence.Copy)
            # menu.addAction(QtGui.QApplication.translate("PythonShell", "Paste"), self.paste, QtGui.QKeySequence.Paste)
            menu.addAction(QtGui.QApplication.translate("PythonShell", "Select all"), self.selectAll, QtGui.QKeySequence.SelectAll)
            # menu.addSeparator()

        else:
            menu.addAction(QtGui.QApplication.translate("PythonShell", "Clear"), self.command_clear)
            menu.addAction(QtGui.QApplication.translate("PythonShell", "Restart"), self.command_restart)
            menu.addSeparator()
            menu.addAction(QtGui.QApplication.translate("PythonShell", "Cut"), self.cut, QtGui.QKeySequence.Cut)
            menu.addAction(QtGui.QApplication.translate("PythonShell", "Copy"), self.copy, QtGui.QKeySequence.Copy)
            menu.addAction(QtGui.QApplication.translate("PythonShell", "Paste"), self.paste, QtGui.QKeySequence.Paste)
            menu.addAction(QtGui.QApplication.translate("PythonShell", "Select all"), self.selectAll, QtGui.QKeySequence.SelectAll)
            # menu.addSeparator()
            #menu.addAction(self.main.actionComment_out_region)

        menu.setStyleSheet("""
        QMenu {
            font-family: inherit;
            font-weight: normal;
            }

        """)

        menu.exec_(event.globalPos())



    #----------------------------------------------------------------------
    def step_font_size(self, delta):
        font = self.font()
        size = font.pointSize()
        if delta > 0: size = size + 1
        else: size = size - 1

        self.setStyleSheet("""
        QPlainTextEdit {
            background-color: #333;
            color: #FFFFFF;
            font-family: mono;
            font-weight: normal;
            font-size: {}dpt;
        }
        """.format(size))


    #----------------------------------------------------------------------
    def set_extra_args(self, *args, **kwargs):
        for key in kwargs:
            setattr(self.shell.statement_module, key, kwargs[key])
        self.extra_args.update(kwargs)


    #----------------------------------------------------------------------
    def set_last_line(self):
        cursor = self.textCursor()
        position = cursor.position()
        plain = self.toPlainText()
        index = plain.rfind("\n")
        if position < (index + len(START) + 1): self.moveCursor(QtGui.QTextCursor.End)


    #----------------------------------------------------------------------
    def no_overwrite_start(self):
        """No move the cursor over >>>"""
        cursor = self.textCursor()
        position = cursor.position()
        plain = self.toPlainText()
        index = plain.rfind("\n")
        if position > index:
            return (position - index) > len(START) + 1
        #if position < index: self.moveCursor(QtGui.QTextCursor.End)


    #----------------------------------------------------------------------
    def run_default_command(self, command):
        try:
            command = command.replace("\n", "")
            run = getattr(self, "command_"+command, None)
            if run: run()
        except:
            return False

        return bool(run)


    #----------------------------------------------------------------------
    def command_clear(self):
        self.clear()
        self.appendPlainText(START)


    #----------------------------------------------------------------------
    def command_restart(self):
        self.shell.restart()
        self.clear()
        self.appendPlainText(HEAD)
        self.appendPlainText(HELP)
        self.set_extra_args(**self.extra_args)
        self.appendPlainText(START)



#! /usr/bin/python
#-*- coding: utf-8 -*-

#import pickle
#import sys
#import re
#import os
#from ConfigParser import RawConfigParser

from PySide import QtGui, QtCore

from .syntax import Autocompleter

########################################################################
class Highlighter(QtGui.QSyntaxHighlighter):

    #----------------------------------------------------------------------
    def __init__(self, parent):
        super(Highlighter, self).__init__(parent)
        color = QtGui.QColor

        self.highlightingRules = []
        self.highlightingRulesMatch = []

        reservadas = QtGui.QTextCharFormat()
        reservadas.setForeground(color("#0000ff"))
        all_reservadas = Autocompleter["reserved"] + Autocompleter["directive"]
        #namespaces = pickle.load(open(os.path.join(os.getenv("PINGUINO_USER_PATH"), "reserved.pickle"), "r"))
        #namespaces = filter(lambda s:not "." in s, namespaces["all"])
        #all_reservadas += namespaces
        self.highlightingRules.append(("\\b("+"|".join(all_reservadas)+")\\b", reservadas))

        dotFuntions = QtGui.QTextCharFormat()
        dotFuntions.setForeground(color("#0000ff"))
        self.highlightingRules.append(("\\b[\D][\w]*\.[\D][\w]*", dotFuntions))

        decimal = QtGui.QTextCharFormat()
        decimal.setForeground(color("#ff0000"))
        self.highlightingRules.append(("\\b[\d]+\\b", decimal))

        float_ = QtGui.QTextCharFormat()
        float_.setForeground(color("#ff0000"))
        self.highlightingRules.append(("\\b[\d]+\.[\d]+\\b", float_))

        hexa = QtGui.QTextCharFormat()
        hexa.setForeground(color("#ff0000"))
        self.highlightingRules.append(("\\b0[Xx][A-Fa-f\d]+\\b", hexa))

        operators = QtGui.QTextCharFormat()
        operators.setFontWeight(QtGui.QFont.Bold)
        self.highlightingRules.append(("[()\[\]{}<>=\-\+\*\\%#!~&^,]", operators))

        bin_ = QtGui.QTextCharFormat()
        bin_.setForeground(color("#ff0000"))
        self.highlightingRules.append(("\\b0[Bb][01]+\\b", bin_))

        doubleQuotation = QtGui.QTextCharFormat()
        doubleQuotation.setForeground(color("#7f0000"))
        self.highlightingRules.append((r'"[^"\\]*(\\.[^"\\]*)*"', doubleQuotation))

        singleQuotation = QtGui.QTextCharFormat()
        singleQuotation.setForeground(color("#cc0000"))
        self.highlightingRules.append((r"'[^'\\]*(\\.[^'\\]*)*'", singleQuotation))

        directives = QtGui.QTextCharFormat()
        directives.setForeground(color("#d36820"))
        self.highlightingRules.append(("#[ ]*[define|include|ifndef|endif|pragma][ ]*.*", directives))

        singleComment = QtGui.QTextCharFormat()
        singleComment.setForeground(color("#007F00"))
        self.highlightingRules.append((r'//[^\n]*', singleComment))

        self.multiComment = QtGui.QTextCharFormat()
        self.multiComment.setForeground(color("#c81818"))

        self.commentStartExpression = QtCore.QRegExp("/\\*")
        self.commentEndExpression = QtCore.QRegExp("\\*/")

    #----------------------------------------------------------------------
    def highlightBlock(self, text):
        for pattern, format_ in self.highlightingRules:
            expression = QtCore.QRegExp(pattern)
            index = expression.indexIn(text)
            while index >= 0:
                length = expression.matchedLength()
                self.setFormat(index, length, format_)
                index = expression.indexIn(text, index + length)

        #for pattern, format in self.highlightingRulesMatch:
            #expression = QtCore.QRegExp(pattern)
            ##index  =  expression.indexIn(text)
            #print expression.capturedTexts()

            #while index >= 0:
                #length = expression.matchedLength()
                #self.setFormat(index, length, format)
                #index = expression.indexIn(text, index + length)

        self.buildComment(self.commentStartExpression,
                          self.commentEndExpression,
                          self.multiComment,
                          text)

    #----------------------------------------------------------------------
    def buildComment(self, start, end, format_, text):
        self.setCurrentBlockState(0)
        startIndex = 0
        if self.previousBlockState() != 1:
            startIndex = start.indexIn(text)
        while startIndex >= 0:
            endIndex = end.indexIn(text, startIndex)
            if endIndex == -1:
                self.setCurrentBlockState(1)
                commentLength = len(text) - startIndex
            else:
                commentLength = endIndex - startIndex + end.matchedLength()
            self.setFormat(startIndex, commentLength, format_)
            startIndex = start.indexIn(text, startIndex + commentLength)

    #----------------------------------------------------------------------
    def addWord(self, word, tipo):
        color = QtGui.QColor
        newWord = QtGui.QTextCharFormat()
        newWord.setForeground(color(*self.fontsTypes(tipo)[0][:3]))
        if self.fontsTypes(tipo)[1]: newWord.setFontWeight(QtGui.QFont.Bold)
        newWord.setFontItalic(self.fontsTypes(tipo)[2])
        self.highlightingRules.insert(0, (QtCore.QRegExp(word), newWord))

    #----------------------------------------------------------------------
    def removeWord(self, word):
        self.highlightingRules.append((QtCore.QRegExp(word), QtGui.QTextCharFormat()))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/yeison/Documentos/Desarrollo/Pinguino/GitHub/pinguino-ide/pinguino/qtgui/gide/bloques/widgets/control_spin.ui'
#
# Created: Tue Nov 17 17:51:41 2015
#      by: pyside-uic 0.2.15 running on PySide 1.2.4
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_Frame(object):
    def setupUi(self, Frame):
        Frame.setObjectName("Frame")
        Frame.resize(94, 32)
        Frame.setWindowTitle("")
        self.gridLayout = QtGui.QGridLayout(Frame)
        self.gridLayout.setContentsMargins(0, 0, 3, 0)
        self.gridLayout.setHorizontalSpacing(3)
        self.gridLayout.setVerticalSpacing(0)
        self.gridLayout.setObjectName("gridLayout")
        self.lineEdit = QtGui.QLineEdit(Frame)
        font = QtGui.QFont()
        font.setFamily("Ubuntu Mono")
        font.setPointSize(15)
        font.setWeight(75)
        font.setBold(True)
        self.lineEdit.setFont(font)
        self.lineEdit.setText("Bloq")
        self.lineEdit.setFrame(False)
        self.lineEdit.setAlignment(QtCore.Qt.AlignCenter)
        self.lineEdit.setObjectName("lineEdit")
        self.gridLayout.addWidget(self.lineEdit, 0, 0, 1, 1)
        self.frame_4 = QtGui.QFrame(Frame)
        self.frame_4.setObjectName("frame_4")
        self.gridLayout_4 = QtGui.QGridLayout(self.frame_4)
        self.gridLayout_4.setSpacing(0)
        self.gridLayout_4.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.verticalLayout_2 = QtGui.QVBoxLayout()
        self.verticalLayout_2.setSpacing(0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.frame = QtGui.QFrame(self.frame_4)
        self.frame.setMinimumSize(QtCore.QSize(15, 15))
        self.frame.setMaximumSize(QtCore.QSize(15, 15))
        self.frame.setObjectName("frame")
        self.verticalLayout_2.addWidget(self.frame)
        self.frame_2 = QtGui.QFrame(self.frame_4)
        self.frame_2.setMinimumSize(QtCore.QSize(15, 15))
        self.frame_2.setMaximumSize(QtCore.QSize(15, 15))
        self.frame_2.setObjectName("frame_2")
        self.verticalLayout_2.addWidget(self.frame_2)
        self.gridLayout_4.addLayout(self.verticalLayout_2, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.frame_4, 0, 1, 1, 1)

        self.retranslateUi(Frame)
        QtCore.QMetaObject.connectSlotsByName(Frame)

    def retranslateUi(self, Frame):
        pass




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/yeison/Documentos/Desarrollo/Pinguino/GitHub/pinguino-ide/pinguino/qtgui/frames/blocks_widget.ui'
#
# Created: Tue Nov 17 17:51:43 2015
#      by: pyside-uic 0.2.15 running on PySide 1.2.4
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_widgetBlock(object):
    def setupUi(self, widgetBlock):
        widgetBlock.setObjectName("widgetBlock")
        widgetBlock.resize(306, 360)
        widgetBlock.setWindowTitle("Tool Area")
        self.gridLayout = QtGui.QGridLayout(widgetBlock)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setSpacing(0)
        self.gridLayout.setObjectName("gridLayout")
        self.scrollArea = QtGui.QScrollArea(widgetBlock)
        self.scrollArea.setFrameShape(QtGui.QFrame.NoFrame)
        self.scrollArea.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.scrollArea.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtGui.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 306, 360))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.gridLayout_6 = QtGui.QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout_6.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_6.setSpacing(0)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.frame_2 = QtGui.QFrame(self.scrollAreaWidgetContents)
        self.frame_2.setFrameShape(QtGui.QFrame.NoFrame)
        self.frame_2.setFrameShadow(QtGui.QFrame.Raised)
        self.frame_2.setObjectName("frame_2")
        self.gridLayout_4 = QtGui.QGridLayout(self.frame_2)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.gridLayout_3 = QtGui.QGridLayout()
        self.gridLayout_3.setSpacing(0)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.frame = QtGui.QFrame(self.frame_2)
        self.frame.setFrameShape(QtGui.QFrame.NoFrame)
        self.frame.setFrameShadow(QtGui.QFrame.Raised)
        self.frame.setObjectName("frame")
        self.gridLayout_5 = QtGui.QGridLayout(self.frame)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.gridLayout_2 = QtGui.QGridLayout()
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.gridLayout_5.addLayout(self.gridLayout_2, 0, 0, 1, 1)
        self.gridLayout_3.addWidget(self.frame, 0, 0, 1, 1)
        self.gridLayout_4.addLayout(self.gridLayout_3, 0, 0, 1, 1)
        spacerItem = QtGui.QSpacerItem(257, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem, 0, 1, 1, 1)
        spacerItem1 = QtGui.QSpacerItem(20, 332, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout_4.addItem(spacerItem1, 1, 0, 1, 1)
        self.gridLayout_6.addWidget(self.frame_2, 0, 0, 1, 1)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.gridLayout.addWidget(self.scrollArea, 0, 0, 1, 1)

        self.retranslateUi(widgetBlock)
        QtCore.QMetaObject.connectSlotsByName(widgetBlock)

    def retranslateUi(self, widgetBlock):
        pass




#! /usr/bin/python
#-*- coding: utf-8 -*- 


#from part import Ui_Form as Floatable
#from inside import Ui_Form as Inside
#from inside2 import Ui_Form as Inside2

#from part_bool import Ui_Form as FloatableBool
#from inside_bool import Ui_Form as InsideBool
#from inside2_bool import Ui_Form as Inside2Bool



#!/usr/bin/env python
#-*- coding: utf-8 -*-

import os
import json

# Python3 compatibility
if os.getenv("PINGUINO_PYTHON") is "3":
    #Python3
    from urllib.request import urlopen
else:
    #Python2
    from urllib import urlopen

from PySide import QtCore, QtGui

from .parser import PinguinoParser


########################################################################
class UpdateAutocompleter(QtCore.QThread):
    """"""
    signal_add_autocompleter = QtCore.Signal(str, str, str)
    signal_rm_autocompleter = QtCore.Signal(str)

    signal_set_variables = QtCore.Signal(list)
    signal_set_directives = QtCore.Signal(list)
    signal_set_functions = QtCore.Signal(list)


    #----------------------------------------------------------------------
    def __init__(self, parent=None):
        """Constructor"""

        # Python3 compatibility
        if os.getenv("PINGUINO_PYTHON") is "3":
            #Python3
            super().__init__(parent)
        else:
            #Python2
            super(UpdateAutocompleter, self).__init__(parent)


    #----------------------------------------------------------------------
    def set_files(self, files):
        """"""
        self.parser = PinguinoParser(files)


    #----------------------------------------------------------------------
    def run(self):

        self.completer_functions = []
        self.functions_args = {}
        functions_parse = self.parser.get_functions()
        self.signal_set_functions.emit(functions_parse)

        for funtion in functions_parse:
            self.completer_functions.append(funtion["name"])
            self.functions_args[funtion["name"]] = "{name}({{{{{args}}}}});".format(**funtion)

        self.completer_variables = []
        variables_parse = self.parser.get_variables()
        self.signal_set_variables.emit(variables_parse)

        for variable in variables_parse:
            self.completer_variables.append([variable["name"], variable["type"]])

        self.completer_directives = []
        directives_parse = self.parser.get_directives()
        self.signal_set_directives.emit(directives_parse)

        for directive in directives_parse:
            self.completer_directives.append(directive["name"])

        self.signal_rm_autocompleter.emit("directives")
        self.signal_rm_autocompleter.emit("variables")
        self.signal_rm_autocompleter.emit("functions")

        for item in self.completer_directives:
            self.signal_add_autocompleter.emit("directives", item, "iconDirectives")

        for item in self.completer_functions:
            self.signal_add_autocompleter.emit("functions", item, "iconFunction")

        for item, type_ in self.completer_variables:
            type_ = type_.split()[-1]
            if type_ in "int float bool byte bool double struct union".split():
                icon = "icon{}".format(type_.capitalize())
            else:
                icon = ""

            self.signal_add_autocompleter.emit("variables", item, icon)



########################################################################
class PinguinoRequest(QtCore.QThread):
    """"""
    signal_message = QtCore.Signal(bool, str)


    #----------------------------------------------------------------------
    def __init__(self, url):
        """Constructor"""
        super(PinguinoRequest, self).__init__(None)
        self.url = url


    #----------------------------------------------------------------------
    def run(self):
        """"""
        try:
            response = urlopen(self.url)
            response = response.read().decode("utf8")
            version = json.loads(response).get("tag_name", None)

            if version.lower().startswith("v"):
                version = version[1:]

            if version:
                local_version = os.environ.get("PINGUINO_VERSION")

                html = """
                    <html>
                      <head/>
                      <body>
                        <p>
                          <b>{}</b><br>
                          A new version of Pinguino IDE is available at:<br>
                          <a href="http://www.pinguino.cc/download"><span style=" text-decoration: underline; color:#2980b9;">http://www.pinguino.cc/download</span></a>
                        </p>
                      </body>
                    </html>""".format(version)

                if version > local_version:
                    self.signal_message.emit(True, html)
                    return

                else:
                    self.signal_message.emit(False, "Your copy of Pinguino IDE is up to date.")

        except Exception as msg:
            self.signal_message.emit(False, "ConnectionError")
            return



#!/usr/bin/env python
#-*- coding: utf-8 -*-

all_sets = {}

#----------------------------------------------------------------------
def extractBlocks(model):

    blocks = list(filter(lambda x:not x.startswith("__"), model.__dict__.keys()))
    instanced = model()
    # blocks.extend(filter(lambda x:not x.startswith("__"), instanced.__dict__.keys()))
    for key in blocks: all_sets[key] = getattr(instanced, key)()


from .pinguino import Pinguino
extractBlocks(Pinguino)

from .math_ import Math_
extractBlocks(Math_)

from .control import Control
extractBlocks(Control)

from .widgets import Widgets
extractBlocks(Widgets)

from .invisible import NoVisible
extractBlocks(NoVisible)

from .user import User
extractBlocks(User)

from .editables import Editables
extractBlocks(Editables)

from .communication import Communication
extractBlocks(Communication)

from .funtions import Functions
extractBlocks(Functions)

from .lcd import LCD
extractBlocks(LCD)

from .stepper import Stepper
extractBlocks(Stepper)

from .servo import Servo
extractBlocks(Servo)






#!/usr/bin/env python
#-*- coding: utf-8 -*-

#import os
#import sys

from PySide import QtGui, QtCore

from .widgets.label import Ui_Frame as Label
from .widgets.control_spin import Ui_Frame as ControlSpin
from .widgets.control_slider import Ui_Frame as ControlSlider
from .widgets.name import Ui_Frame as Name
from .inside.inside import Ui_Form as Inside
from .inside.inside2 import Ui_Form as Inside2
from .inside.inside_bool import Ui_Form as InsideBool
from .inside.inside2_bool import Ui_Form as Inside2Bool


FIX_SYNTAX = ", ; = > < >= <= + -".split()

FONT_SIZE = 15
ADJ = 10  #space at end
ADJ2 = -5  #space on expand


#----------------------------------------------------------------------
def changeFontSize(widget):
    #name = widget.__str__()
    #name = name[name.rfind(".")+1:name.find(" ")]

    widget.setStyleSheet("""
    QLineEdit{
        color: #FFFFFF;
        font-family: mono;
        font-weight: normal;
        font-size: 12pt;
    }

    QLabel{
        color: #FFFFFF;
        font-family: mono;
        font-weight: normal;
        font-size: 12pt;
    }

    QPlainTextEdit{
        color: #FFFFFF;
        font-family: mono;
        font-weight: normal;
        font-size: 12pt;
    }

    """)



########################################################################
class Constructor(object):

    DeletLater = False

    def buildBlock(self, widget, bloque, insidePos=(27, 2), inside=False):
        self.layout_adds = []
        self.layout_adds_b = []
        self.layout_adds_all = []
        layout_pos = []
        self.constructorCode = []
        self.LineCode = []
        self.fullWidgetWith = []
        self.codeStart = {}

        changeFontSize(widget)

        for add in bloque:
            if add[0] == "label": self.addLabel(widget, add[1])
            if add[0] == "space": self.addToInside(widget, inside)
            if add[0] == "space_bool": self.addToInsideBool(widget, inside)
            if add[0] == "edit": self.addLineEdit(widget, add[:])
            if add[0] == "spin_int": self.addSpinInt(widget, add[:])
            if add[0] == "spin_choice": self.addChoice(widget, add[:])
            if add[0] == "spin_float": self.addSpinFloat(widget, add[:])
            if add[0] == "syntax": self.addSyntax(widget, add[1])
            if add[0] == "slider": self.addSlider(widget, add[:])
            #if add[0] == "button": self.addButton(widget, add[:])
            if add[0] == "decorator": self.addDecorator(widget, add[1])
            if add[0] == "help": widget.setToolTip(add[1])
            if add[0] == "full": self.fullWidgetWith = add[1]
            if add[0] == "code": self.codeStart = add[1]


        self.setFlex(self.layout_adds, insidePos, self.layout_adds_b, self.layout_adds_all)
        widget.contextMenuEvent = self.build_menu


    #----------------------------------------------------------------------
    def get_widgets_from_layout(self):
        if self.layout == None: return[]
        widgets = []
        for index in range(self.layout.count()):
            widgets.append((index, self.layout.itemAt(index).widget()))

        return filter(lambda wid:wid[1].objectName().startswith("BLOCK"), widgets)


    ##----------------------------------------------------------------------
    #def get_widgets_from_layout_save(self):
        #if self.layout == None: return[]
        #widgets = []
        #for index in range(self.layout.count()):
            #widgets.append((index, self.layout.itemAt(index).widget()))

        #return filter(lambda wid:wid[1].objectName().startswith("BLOCK"), widgets)




    #----------------------------------------------------------------------
    def buildLabel(self, widget, text="Label", rgb=None):
        widgetLabel = QtGui.QWidget(widget)
        label = Label()
        label.setupUi(widgetLabel)
        label.label.setText(text)

        #changeFontSize(label.label)

        if rgb != None:
            #label.label.setStyleSheet("background-color: rgb(%d, %d, %d);\ncolor: rgb(255, 255, 255);" %rgb)
            label.label.setStyleSheet(label.label.styleSheet()+"""
            QLabel{
                background-color: rgb(%d, %d, %d);
                color: rgb(255, 255, 255);
            }

            """%rgb)

        if len(text) > 3:
            widgetLabel.setMinimumSize(len(text)*(FONT_SIZE+ADJ2)+ADJ, widgetLabel.size().height())
        else:
            widgetLabel.setMinimumSize(len(text)*(FONT_SIZE+ADJ2), widgetLabel.size().height())

        self.LineCode.append(lambda :text)
        return widgetLabel, label.label

    #----------------------------------------------------------------------
    def buildDecorator(self, widget, text="Label", rgb=None):
        widgetLabel = QtGui.QWidget(widget)
        label = Label()
        label.setupUi(widgetLabel)
        label.label.setText(text)
        #changeFontSize(label.label)
        if rgb != None:
            #label.label.setStyleSheet("background-color: rgb(%d, %d, %d);\ncolor: rgb(255, 255, 255);" %rgb)            label.label.setStyleSheet(label.label.styleSheet()+"""
            label.label.setStyleSheet(label.label.styleSheet()+"""
            QLabel{
                background-color: rgb(%d, %d, %d);
                color: rgb(255, 255, 255);
            }

            """%rgb)

        if len(text) > 3:
            widgetLabel.setMinimumSize(len(text)*(FONT_SIZE+ADJ2)+ADJ, widgetLabel.size().height())
        else:
            widgetLabel.setMinimumSize(len(text)*(FONT_SIZE+ADJ2), widgetLabel.size().height())

        return widgetLabel, label.label


    #----------------------------------------------------------------------
    def buildSpinInt(self, widget, start, b_min=-100, b_max=100):
        widgetSpin = QtGui.QWidget(widget)
        spin = ControlSpin()
        spin.setupUi(widgetSpin)

        #icon = QtGui.QIcon()
        #icon.addPixmap(QtGui.QPixmap("%sup.png"%self.isideDir), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        #spin.toolButton_3.setIcon(icon)

        #icon1 = QtGui.QIcon()
        #icon1.addPixmap(QtGui.QPixmap("%sdown.png"%self.isideDir), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        #spin.toolButton_4.setIcon(icon1)


        spin.frame.setStyleSheet("image: url(%sup.png);"%self.isideDir)
        spin.frame_2.setStyleSheet("image: url(%sdown.png);"%self.isideDir)

        edit = spin.lineEdit
        #edit.setStyleSheet("background-image: url(:/bg/bg/box.png);\ncolor: rgb(90, 90, 90);")

        b_up = spin.frame
        b_down = spin.frame_2

        #changeFontSize(edit)

        edit.setStyleSheet(edit.styleSheet()+"""
        QLineEdit{
            background-image: url(:/bg/bg/box.png);
            color: rgb(90, 90, 90);
        }

        """)


        edit.setText(str(start))

        def up(*args):
            try: current = int(edit.text())
            except ValueError: current = start
            if current == b_max: b_next = str(b_min)
            else: b_next = str(current+1)
            edit.setText(b_next)

        def down(*args):
            try: current = int(edit.text())
            except ValueError: current = start
            if current == b_min: b_next = str(b_max)
            else: b_next = str(current-1)
            edit.setText(b_next)

        b_up.setCursor(QtCore.Qt.PointingHandCursor)
        b_down.setCursor(QtCore.Qt.PointingHandCursor)
        b_up.mousePressEvent = up
        b_down.mousePressEvent = down

        edit.setMinimumSize(50, 25)
        edit.setMaximumSize(50, 25)

        self.LineCode.append(edit.text)
        return widgetSpin, edit


    #----------------------------------------------------------------------
    def buildSpinFloat(self, widget, start, step=0.1, b_min=-100.0, b_max=100.0):
        widgetSpin = QtGui.QWidget(widget)
        spin = ControlSpin()
        spin.setupUi(widgetSpin)

        #icon = QtGui.QIcon()
        #icon.addPixmap(QtGui.QPixmap("%sup.png"%self.isideDir), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        #spin.toolButton_3.setIcon(icon)

        #icon1 = QtGui.QIcon()
        #icon1.addPixmap(QtGui.QPixmap("%sdown.png"%self.isideDir), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        #spin.toolButton_4.setIcon(icon1)


        spin.frame.setStyleSheet("image: url(%sup.png);"%self.isideDir)
        spin.frame_2.setStyleSheet("image: url(%sdown.png);"%self.isideDir)


        edit = spin.lineEdit
        b_up = spin.frame
        b_down = spin.frame_2

        edit.setText(str(start))
        #changeFontSize(edit)

        edit.setStyleSheet(edit.styleSheet()+"""
        QLineEdit{
            background-image: url(:/bg/bg/box.png);
            color: rgb(90, 90, 90);
        }

        """)


        def up(*args):
            try: current = float(edit.text())
            except ValueError: current = start
            if current == b_max: b_next = str(b_min)
            else: b_next = str(current+step)
            edit.setText(b_next)

        def down(*args):
            try: current = float(edit.text())
            except ValueError: current = start
            if current == b_min: b_next = str(b_max)
            else: b_next = str(current-step)
            edit.setText(b_next)


        b_up.setCursor(QtCore.Qt.PointingHandCursor)
        b_down.setCursor(QtCore.Qt.PointingHandCursor)
        b_up.mousePressEvent = up
        b_down.mousePressEvent = down


        edit.setMinimumSize(0, 25)
        edit.setMaximumSize(0, 25)

        self.LineCode.append(edit.text)
        return widgetSpin, edit


    #----------------------------------------------------------------------
    def buildName(self, widget, text, background=None, color=(255, 255, 255)):
        widgetName = QtGui.QWidget(widget)
        name = Name()
        name.setupUi(widgetName)
        name.lineEdit.setText(str(text))
        #changeFontSize(name.lineEdit)

        if background == "white" and color != None:
            #name.lineEdit.setStyleSheet("background-image: url(:/bg/bg/box.png);\ncolor: rgb(%d, %d, %d);" %color)
            name.lineEdit.h = name.lineEdit.size().height() - 5

            name.lineEdit.setStyleSheet("""
            QLineEdit{
                background-image: url(:/bg/bg/box.png);
                color: rgb(%d, %d, %d);
                font-family: mono;
            }
            """%color)




        elif background == "white" and color == None:
            #name.lineEdit.setStyleSheet("background-image: url(:/bg/bg/box.png);")
            name.lineEdit.h = name.lineEdit.size().height() - 5
            name.lineEdit.setStyleSheet("""
            QLineEdit{
                background-image: url(:/bg/bg/box.png);
                font-family: mono;
            }
            """)



        elif background == None and color != None:
            #name.lineEdit.setStyleSheet("color: rgb(%d, %d, %d);" %color)
            name.lineEdit.h = name.lineEdit.size().height() + 4
            name.lineEdit.setStyleSheet("""
            QLineEdit{
                color: rgb(%d, %d, %d);
                font-family: mono;
            }
            """%color)


        self.LineCode.append(name.lineEdit.text)
        return widgetName, name.lineEdit


    #----------------------------------------------------------------------
    def buildChoice(self, widget, curr, opc):

        opc = list(opc)
        widgetChoice = QtGui.QWidget(widget)
        choice = ControlSpin()
        choice.setupUi(widgetChoice)
        choice.lineEdit.setText(curr)
        choice.lineEdit.setReadOnly(True)
        #choice.lineEdit.sele(True)

        choice.lineEdit.setStyleSheet("background-image: url(:/bg/bg/box.png); color: rgb(90,90,90);")

        #icon = QtGui.QIcon()
        #icon.addPixmap(QtGui.QPixmap("%sup.png"%self.isideDir), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        #choice.toolButton_3.setIcon(icon)

        choice.frame.setStyleSheet("image: url(%sup.png);"%self.isideDir)
        choice.frame_2.setStyleSheet("image: url(%sdown.png);"%self.isideDir)


        edit = choice.lineEdit
        b_up = choice.frame
        b_down = choice.frame_2
        b_min = 0
        b_max = len(opc) - 1

        #changeFontSize(edit)


        def up(*args):
            try: current = opc.index(edit.text())
            except ValueError: current = 0
            if current == b_max: b_next = b_min
            else: b_next = current+1
            edit.setText(opc[b_next])

        def down(*args):
            try: current = opc.index(edit.text())
            except ValueError: current = 0
            if current == b_min: b_next = b_max
            else: b_next = current-1
            edit.setText(opc[b_next])

        b_up.setCursor(QtCore.Qt.PointingHandCursor)
        b_down.setCursor(QtCore.Qt.PointingHandCursor)
        b_up.mousePressEvent = up
        b_down.mousePressEvent = down

        #widgetLabel.setMinimumSize(len(text)*FONT_SIZE+ADJ, widgetLabel.size().height())

        l = 0
        for ch in opc:
            if len(ch) > l: l = len(ch)


        edit.setMinimumSize(l*(FONT_SIZE+ADJ2)+20, 25)
        edit.setMaximumSize(l*(FONT_SIZE+ADJ2)+20, 25)

        self.LineCode.append(edit.text)
        return widgetChoice, edit

    #----------------------------------------------------------------------
    def buildSliderInt(self, widget, start, b_min=0, b_max=1023):
        widgetSlider = QtGui.QWidget(widget)
        slider = ControlSlider()
        slider.setupUi(widgetSlider)

        edit = slider.lineEdit_2
        #changeFontSize(edit)
        slid = slider.horizontalSlider
        #edit.setStyleSheet("background-image: url(:/general/arte/box.png);\ncolor: rgb(0, 0, 0);")
        #edit.setStyleSheet("color: rgb(255, 255, 255);")
        edit.setStyleSheet(edit.styleSheet()+"""
        QLineEdit {
        color: #FFFFFF;
        }

        """)


        def updateEdit(value): edit.setText(str(value))
        QtCore.QObject.connect(slid, QtCore.SIGNAL("sliderMoved(int)"), updateEdit)
        edit.setText(str(start))
        slid.setValue(int(start))

        widgetSlider.setMinimumSize(180, widgetSlider.size().height())
        widgetSlider.setMaximumSize(180, widgetSlider.size().height())

        self.LineCode.append(edit.text)
        return widgetSlider, edit

    #----------------------------------------------------------------------
    def buidToInside(self, widget, insideType):
        widgetInside = QtGui.QWidget(widget)
        widgetInside.getLine = lambda :""
        if insideType:
            inside = Inside2()
            inside.setupUi(widgetInside)
            widgetInside.setMinimumHeight(34)
            widgetInside.setMaximumHeight(34)
            inside.frame.setStyleSheet("image: url(%s/insert21b.png);"%self.isideDir)
            inside.frame_2.setStyleSheet("background-image: url(%s/insert23b.png);"%self.isideDir)
            #inside.frame_3.setStyleSheet("image: url(bloques/inside/arte/insert22b.png);")


        else:
            inside = Inside()
            inside.setupUi(widgetInside)

            inside.frame.setStyleSheet("image: url(%s/insert21.png);"%self.isideDir)
            inside.frame_2.setStyleSheet("background-image: url(%s/insert23.png);"%self.isideDir)



        widgetInside.setMinimumSize(30, widgetInside.size().height())
        widgetInside.setMaximumSize(30, widgetInside.size().height())

        self.LineCode.append(lambda :"%s")
        return widgetInside



    #----------------------------------------------------------------------
    def buidToInsideBool(self, widget, insideType):
        widgetInside = QtGui.QWidget(widget)
        widgetInside.getLine = lambda :""
        if insideType:
            inside = Inside2Bool()
            inside.setupUi(widgetInside)
            widgetInside.setMinimumHeight(34)
            widgetInside.setMaximumHeight(34)

            inside.frame.setStyleSheet("image: url(%s/insert21b.png);"%self.isideDir)
            inside.frame_2.setStyleSheet("background-image: url(%s/insert23b.png);"%self.isideDir)
            inside.frame_3.setStyleSheet("image: url(%s/insert21b_.png);"%self.isideDir)

        else:
            inside = InsideBool()
            inside.setupUi(widgetInside)
            inside.frame.setStyleSheet("image: url(%s/insert21.png);"%self.isideDir)
            inside.frame_2.setStyleSheet("background-image: url(%s/insert23.png);"%self.isideDir)
            #inside.frame_3.setStyleSheet("image: url(%sinsert22.png);"%self.isideDir)
            inside.frame_3.setStyleSheet("image: url(%s/insert21_.png);"%self.isideDir)


        widgetInside.setMinimumSize(30, widgetInside.size().height())
        widgetInside.setMaximumSize(30, widgetInside.size().height())

        self.LineCode.append(lambda :"%s")
        return widgetInside


    #----------------------------------------------------------------------
    def addParent(self, widget, force=False):
        static, move = widget
        static_layout = static.metadata.object_.layout

        inside_pos, layout_pos = static.metadata.object_.getInsidePos()

        if not force:
            for ins in inside_pos:
                c = ins - move.parent().mapToGlobal(move.pos())
                if abs(c.x()) < 4 and  abs(c.y()) < 4:
                    index = inside_pos.index(ins)
        else:
            index = 0

        static_layout.itemAt(layout_pos[index]).widget().hide()
        static_layout.itemAt(layout_pos[index]).widget().setEnabled(False)
        static_layout.insertWidget(layout_pos[index], move)

        #update parents
        if static.metadata.parent == "None":
            move.metadata.parent = static
        else: move.metadata.parent = static.metadata.parent

        #updtate iside
        static.metadata.inside.append(move)

        #update linecode
        move.getLine = move.metadata.line_code

        #fit heigth
        move.setMinimumHeight(move.height())
        move.setMaximumHeight(move.height())

        self.remove = (
                       #static_layout.itemAt(layout_pos[index]).widget(),
                       static_layout.itemAt(layout_pos[index]+1).widget(),
                       static,
                       move,
                       )


        ##pasado = WidgetStatic.metadata.object_.getWidgtes()[r]
        #pasado = WidgetStatic.metadata.object_.getWidgtes()[r]

        #pasado.hide()
        #self.pasado = pasado
        ##self.layoutPos = layoutPos[r]
        ##WidgetStatic.metadata.inside[r] = widgetMove.ID
        #self.w_parent = WidgetStatic
        #self.w_move = widgetMove
        #self.R = r
        #self.updateSize()
        ##self.w_parent.adjustSize()

        ##self.toAddPoint = WidgetStatic.metadata.object_.inLayouts_all.pop(r)
        ##self.toAddPos = WidgetStatic.metadata.object_.layoutsPos.pop(r)
        #self.toAddPoint = WidgetStatic.metadata.object_.inLayouts_all.pop(r)
        #self.toAddPos = WidgetStatic.metadata.object_.layoutsPos.pop(r)






        ##self.primo = widget
   #-        widgetMove = widget[1]
   #-        self.widgetMove = widgetMove
   #-        WidgetStatic = widget[0]
   #-
   #-        r = 0
   #-        #insidePos, layoutPos = WidgetStatic.metadata.object_.getInsidePos()
   #-        insidePos, layoutPos = WidgetStatic.metadata.object_.getInsidePos()
   #-        #print layoutPos
   #-        for po in insidePos:
   #-
   #-            if not force:
   #-                a = widgetMove.pos()
   #-                #b = po + WidgetStatic.DATA["pos"] + WidgetStatic.metadata.object_.getInsidePoint()
   #-                b = po + WidgetStatic.metadata.pos_ + WidgetStatic.metadata.object_.getInsidePoint()
   #-                poGlobal = po
   #-                aGlobal = widgetMove.parent().mapToGlobal(a)
   #-                c = poGlobal - aGlobal
   #-            else:
   #-                r = 0
   #-                c = QtCore.QPoint()
   #-
   #-            #print c.x()
   #-            if -4 <= c.x() <= 4:
   #-
   #-
   #-                if WidgetStatic.metadata.parent == "None":
   #-                    widgetMove.metadata.parent = WidgetStatic
   #-                else: widgetMove.metadata.parent = WidgetStatic.metadata.parent
   #-
   #-                if layoutPos[r] == layoutPos[-1]:
   #-                    WidgetStatic.metadata.object_.layout.insertWidget(layoutPos[r]+1, widgetMove)
   #-
   #-                else:
   #-                    WidgetStatic.metadata.object_.layout.insertWidget(layoutPos[r], widgetMove)
   #-
   #-                WidgetStatic.metadata.inside.append(widgetMove)
   #-
   #-
   #-
   #-                #widgetMove.getLine = widgetMove.DATA["lineCode"]
   #-                widgetMove.getLine = widgetMove.metadata.line_code
   #-
   #-
   #-                widgetMove.setMinimumHeight(widgetMove.height())
   #-                widgetMove.setMaximumHeight(widgetMove.height())
   #-
   #-                #pasado = WidgetStatic.metadata.object_.getWidgtes()[r]
   #-                pasado = WidgetStatic.metadata.object_.getWidgtes()[r]
   #-
   #-                pasado.hide()
   #-                self.pasado = pasado
   #-                #self.layoutPos = layoutPos[r]
   #-                #WidgetStatic.metadata.inside[r] = widgetMove.ID
   #-                self.w_parent = WidgetStatic
   #-                self.w_move = widgetMove
   #-                self.R = r
   #-                self.updateSize()
   #-                #self.w_parent.adjustSize()
   #-
   #-                #self.toAddPoint = WidgetStatic.metadata.object_.inLayouts_all.pop(r)
   #-                #self.toAddPos = WidgetStatic.metadata.object_.layoutsPos.pop(r)
   #-                self.toAddPoint = WidgetStatic.metadata.object_.inLayouts_all.pop(r)
   #-                self.toAddPos = WidgetStatic.metadata.object_.layoutsPos.pop(r)
   #-
   #-                return
   #-
   #-            r += 1




    #----------------------------------------------------------------------
    def removeParent(self):
        hidden, static, move = self.remove

        static.metadata.parent = "None"
        move.metadata.parent = "None"


        #static.metadata.object_.inLayouts_all.insert(self.R, self.toAddPoint)
        #static.metadata.object_.layoutsPos.insert(self.R, self.toAddPos)
        hidden.show()
        hidden.setEnabled(True)
        static.metadata.object_.layout.removeWidget(static)




    #----------------------------------------------------------------------
    def updatePoints(self):
        layouts = filter(lambda l:l.isEnabled(), self.inLayouts)
        points = map(lambda x:self.widget.mapToGlobal(x.pos()+self.insidePos), layouts)
        self.toType["tipo5"] = points

        layouts_b = filter(lambda l:l.isEnabled(), self.inLayouts_b)
        points = map(lambda x:self.widget.mapToGlobal(x.pos()+self.insidePos), layouts_b)
        self.toType["tipo2"] = points


    #----------------------------------------------------------------------
    def getInsidePos(self):
        #intros = [map(lambda x:self.widget.mapToGlobal(x.pos()+self.insidePos), self.inLayouts_all),
                  #self.get_layout_pos()]
        intros = [map(lambda x:self.widget.mapToGlobal(x.pos()+self.insidePos), self.get_layout_widgets()),
                  self.get_layout_pos()]
        return intros

    ##----------------------------------------------------------------------
    #def getWidgtes(self):
        #return self.inLayouts_all

    #----------------------------------------------------------------------
    def get_layout_pos(self):
        widgets = self.get_widgets_from_layout()
        layoutpos = []
        for index, wid in widgets:
            if wid.objectName() == "BLOCK_SPACE" and wid.isEnabled():
                layoutpos.append(index)
        return layoutpos

    #----------------------------------------------------------------------
    def get_layout_pos_save(self):
        widgets = self.get_widgets_from_layout()
        layoutpos = []
        for index, wid in widgets:
            if wid.objectName() == "BLOCK_SPACE":
                layoutpos.append(wid.isHidden())
            #elif wid.objectName().startswith("BLOCK"):
                #layoutpos.append
        return layoutpos

    #----------------------------------------------------------------------
    def get_layout_widgets(self):
        widgets = self.get_widgets_from_layout()
        layoutpos = []
        for index, wid in widgets:
            if wid.objectName() == "BLOCK_SPACE" and wid.isEnabled():
                layoutpos.append(wid)
        return layoutpos

    #----------------------------------------------------------------------
    def setFlex(self, inLayouts, insidePos, inLayouts_b, inlayouts_all):
        #self.layoutsPos = layoutsPos
        self.inLayouts = inLayouts
        self.insidePos = QtCore.QPoint(*insidePos)

        self.inLayouts_b = inLayouts_b
        self.inLayouts_all = inlayouts_all


    #----------------------------------------------------------------------
    def getInsidePoint(self):
        try: return self.insidePos
        except: return QtCore.QPoint(27, 2)

    def addChoice(self, widget, add):
        widget_choide, lineEdit = self.buildChoice(*[widget] + add[1:])
        widget_choide.setObjectName("BLOCK_CHOICE")
        self.layout.addWidget(widget_choide)
        lineEdit.contextMenuEvent = self.build_menu
        # self.set_as_draggable(lineEdit, widget)
        widget.set_value = lambda value:lineEdit.setText(value)
        self.constructorCode.append(lambda :["spin_choice"]+[str(lineEdit.text())]+add[2:])

    def addSpinInt(self, widget, add):
        widgetSpin, lineEdit = self.buildSpinInt(*[widget] + add[1:])
        widgetSpin.setObjectName("BLOCK_SPININT")
        self.layout.addWidget(widgetSpin)
        lineEdit.h = lineEdit.size().height()
        QtCore.QObject.connect(lineEdit, QtCore.SIGNAL("textChanged(QString)"), lambda x:self.updateLine(lineEdit))
        self.updateLine(lineEdit)
        lineEdit.contextMenuEvent = self.build_menu
        widget.set_value = lambda value:lineEdit.setText(value)
        # self.set_as_draggable(lineEdit, widget)
        self.constructorCode.append(lambda :["spin_int"]+[str(lineEdit.text())]+add[2:])

    def addSpinFloat(self, widget, add):
        widgetSpin, lineEdit = self.buildSpinFloat(*[widget] + add[1:])
        widgetSpin.setObjectName("BLOCK_SPINFLOAT")
        self.layout.addWidget(widgetSpin)
        lineEdit.h = lineEdit.size().height()
        QtCore.QObject.connect(lineEdit, QtCore.SIGNAL("textChanged(QString)"), lambda x:self.updateLine(lineEdit))
        self.updateLine(lineEdit)
        lineEdit.contextMenuEvent = self.build_menu
        widget.set_value = lambda value:lineEdit.setText(value)
        # self.set_as_draggable(lineEdit, widget)
        self.constructorCode.append(lambda :["spin_float", str(lineEdit.text())]+add[2:])

    def addLineEdit(self, widget, add):
        widgetEdit, lineEdit = self.buildName(*[widget] + add[1:])
        widgetEdit.setObjectName("BLOCK_LINEDIT")
        self.layout.addWidget(widgetEdit)
        QtCore.QObject.connect(lineEdit, QtCore.SIGNAL("textChanged(QString)"), lambda x:self.updateLine(lineEdit))
        self.updateLine(lineEdit)
        widget.set_value = lambda value:lineEdit.setText(value)
        # self.set_as_draggable(lineEdit, widget)
        self.constructorCode.append(lambda :["edit", str(lineEdit.text())] + add[2:])
        # lineEdit.setCursor(QtCore.Qt.ArrowCursor)

    def addSlider(self, widget, add):
        widgetSlider, lineEdit = self.buildSliderInt(*[widget] + add[1:])
        widgetSlider.setObjectName("BLOCK_SLIDER")
        self.layout.addWidget(widgetSlider)
        lineEdit.contextMenuEvent = self.build_menu
        widget.set_value = lambda value:lineEdit.setText(value)
        # self.set_as_draggable(lineEdit, widget)
        self.constructorCode.append(lambda :["slider", str(lineEdit.text())])

    #def addButton(self, widget, add):
        #widgetButton, button = self.buildButton(*[widget] + add[1:])
        #widgetButton.setObjectName("BLOCK_BUTOON")
        #self.layout.addWidget(widgetButton)
        #self.constructorCode.append(lambda :["button"]+add[1:])

    def addLabel(self, widget, text):
        widgetLabel, Label = self.buildLabel(widget, str(text))
        widgetLabel.setObjectName("BLOCK_LABEL")
        self.layout.addWidget(widgetLabel)
        self.constructorCode.append(lambda :["label", str(text)])

    def addDecorator(self, widget, text):
        widgetLabel, Label = self.buildDecorator(widget, str(text))
        widgetLabel.setObjectName("BLOCK_DECORATOR")
        self.layout.addWidget(widgetLabel)
        self.constructorCode.append(lambda :["decorator", str(text)])

    def addToInside(self, widget, inside):
        toInside = self.buidToInside(widget, inside)
        setattr(toInside, "in_type", "space")
        toInside.setObjectName("BLOCK_SPACE")
        self.layout.addWidget(toInside)
        self.layout_adds.append(toInside)
        self.layout_adds_all.append(toInside)
        self.constructorCode.append(lambda :["space"])

    def addToInsideBool(self, widget, inside):
        toInsideBool = self.buidToInsideBool(widget, inside)
        setattr(toInsideBool, "in_type", "space")
        toInsideBool.setObjectName("BLOCK_SPACE")
        self.layout.addWidget(toInsideBool)
        self.layout_adds_b.append(toInsideBool)
        self.layout_adds_all.append(toInsideBool)
        self.constructorCode.append(lambda :["space_bool"])

    def addSyntax(self, widget, text):
        for char in FIX_SYNTAX:
            if text == char: text += " "

        widgetLabel, Label = self.buildLabel(widget, "")
        widgetLabel.setObjectName("BLOCK_SYNTAX")
        self.layout.addWidget(widgetLabel)

        self.LineCode.pop(-1)
        self.LineCode.append(lambda :str(text))
        self.constructorCode.append(lambda :["syntax", str(text)])


    def updateLine(self, lineEdit):
        lineEdit.setMinimumSize(len(lineEdit.text())*(FONT_SIZE+ADJ2)+ADJ, lineEdit.h)
        lineEdit.setMaximumSize(len(lineEdit.text())*(FONT_SIZE+ADJ2)+ADJ, lineEdit.h)


    ##----------------------------------------------------------------------
    #def fullWidgetWith(self, content):
        #
        #self.IDE.ge
        #self.addParent([self.getWorkArea().getWidget("0"),self.getWorkArea().getWidget("1")],
                       #force = True, position = 1)




    #----------------------------------------------------------------------
    def getLine(self):

        code = map(lambda x:str(x()), self.LineCode)

        complete = []
        for i in range(self.layout.count()):
            c = self.layout.itemAt(i).widget()
            try:
                if c.getLine() != "": complete.append(c.getLine().replace("\n", ""))
            except: pass

        code = "".join(code)

        if complete:
            code = code % tuple(complete + ["%s"]*(code.count("%s")-len(complete)))

        return code + self.endLine + "\n"



    #----------------------------------------------------------------------
    def getConstructor(self):

        constr = []
        for code in self.constructorCode:
            constr.append(code())
        return constr[:]

    #----------------------------------------------------------------------
    def setAumentWidth(self):
        self.widget.setMinimumSize(self.widget.size()+QtCore.QSize(100, 0))
        self.widget.setMaximumSize(self.widget.size()+QtCore.QSize(100, 0))

    #----------------------------------------------------------------------
    def setAumentHigth(self):
        self.widget.setMinimumSize(self.widget.size()+QtCore.QSize(0, 100))
        self.widget.setMaximumSize(self.widget.size()+QtCore.QSize(0, 100))

    #----------------------------------------------------------------------
    def setDecrementWidth(self):
        self.widget.setMinimumSize(self.widget.size()-QtCore.QSize(100, 0))
        self.widget.setMaximumSize(self.widget.size()-QtCore.QSize(100, 0))

    #----------------------------------------------------------------------
    def setDecrementHigth(self):
        self.widget.setMinimumSize(self.widget.size()-QtCore.QSize(0, 100))
        self.widget.setMaximumSize(self.widget.size()-QtCore.QSize(0, 100))

    # #----------------------------------------------------------------------
    # def set_as_draggable(self, lineEdit, widget):
        # """"""
        # lineEdit.contextMenuEvent = self.build_menu
        # lineEdit.mouseMoveEvent = widget.mouseMoveEvent
        # lineEdit.mousePressEvent = widget.mousePressEvent
        # lineEdit.mouseReleaseEvent = widget.mouseReleaseEvent
        # lineEdit.setCursor(QtCore.Qt.OpenHandCursor)



#!/usr/bin/env python
#-*- coding: utf-8 -*-

import os
from setuptools import setup, find_packages


if os.name == "posix": #GNU/Linux

    os_kwargs = {"data_files": [
                                ("/usr/share/applications", ["pinguino12.desktop"]),
                                ("/usr/share/icons/hicolor/256x256", ["pinguino/qtgui/resources/art/pinguino_logo-256x256.png"]),
                                ("/usr/share/icons/hicolor/256x256", ["pinguino/qtgui/resources/art/pinguino_logo_background_blue-256x256.png"]),
                                ],
                }

elif os.name == "os2":  #Mac OS X
    os_kwargs = {}


elif os.name == "nt":  #Windows
    os_kwargs = {}






setup(name = "pinguino",
      version = "12.0.15",
      description = "Open Hardware Electronics Prototyping Platform, Open Source Integrated Development Environment (IDE)",
      description_file = "README.rst",

      author = "",
      author_email = "",
      maintainer = "Yeison Cardona",
      maintainer_email = "yeison.eng@gmail.com",

      url = "http://www.pinguino.cc/",
      download_url = "",

      license = "GPLv2",
      keywords = "microchip, electronic, prototyping, IDE",

      classifiers=[#list of classifiers in https://pypi.python.org/pypi?:action=list_classifiers
                   "Environment :: X11 Applications :: Qt",
                   "Intended Audience :: Developers",
                   "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
                   "Operating System :: POSIX :: Linux",
                   "Programming Language :: Python",
                   ],

      packages = find_packages(),
      include_package_data = True,

      install_requires = [
                          "gitpython",
                          "hgapi",
                          "pyusb==1.0.0b2",
                          "pyside",
                          ],

      scripts = [
                 "bin/pinguino",
                 "bin/pinguino.pyw",
                 "bin/pinguino-cmd",
                 "bin/pinguino-cmd.py",
                 "bin/pinguino-reset",
                 "bin/pinguino-reset.py",
                 ],

      zip_safe = False,
      **os_kwargs

      )



#!/usr/bin/env python
#-*- coding: utf-8 -*-
#-*- coding: utf-8 -*-

import usb
import sys
import logging
import os
#import debugger

# from .. import Debugger

########################################################################
class baseUploader(object):

    # Hex format record types
    # --------------------------------------------------------------------------
    # Data_Record = 00
    # End_Of_File_Record = 01
    # Extended_Segment_Address_Record = 02
    # Start_Segment_Address_Record = 03
    # Extended_Linear_Address_Record = 04
    # Start_Linear_Address_Record = 05

    # Python3 compatibility (octals)
    Data_Record                     = 0o0
    End_Of_File_Record              = 0o1
    Extended_Segment_Address_Record = 0o2
    Start_Segment_Address_Record    = 0o3
    Extended_Linear_Address_Record  = 0o4
    Start_Linear_Address_Record     = 0o5


    # Error codes returned by various functions
    # --------------------------------------------------------------------------
    ERR_NONE                        = 100
    ERR_CMD_ARG                     = 101
    ERR_CMD_UNKNOWN                 = 102
    ERR_DEVICE_NOT_FOUND            = 103
    ERR_USB_INIT1                   = 104
    ERR_USB_INIT2                   = 105
    ERR_USB_OPEN                    = 106
    ERR_USB_WRITE                   = 107
    ERR_USB_READ                    = 108
    ERR_HEX_OPEN                    = 109
    ERR_HEX_STAT                    = 110
    ERR_HEX_MMAP                    = 111
    ERR_HEX_SYNTAX                  = 112
    ERR_HEX_CHECKSUM                = 113
    ERR_HEX_RECORD                  = 114
    ERR_VERIFY                      = 115
    ERR_EOL                         = 116
    ERR_USB_ERASE                   = 117

    # Configuration
    # ----------------------------------------------------------------------

    INTERFACE_ID                    = 0x00
    ACTIVE_CONFIG                   = 0x01
    VSC_ACTIVE_CONFIG               = 0x02

# # ------------------------------------------------------------------------------
    # def __init__(self, hex_file, board):
        # self.hex_file = hex_file
        # self.filename = hex_file
        # self.board = board
        # self.report = []


# ------------------------------------------------------------------------------
    def add_report(self, message):
        """ display message in the log window """
        self.report.append(message)

        #import sys
        #reload(sys)
        #sys.stdout.write("DEBUG : " + message + "\r\n")
        logging.info(message)

# # ------------------------------------------------------------------------------
    # def getDevice(self):
        # """ Get a list of USB devices and search for a Pinguino board """
        # busses = usb.busses()
        # for bus in busses:
            # for device in bus.devices:
                # if device.idVendor == self.board.vendor and device.idProduct == self.board.product:
                    # return device
        # return self.ERR_DEVICE_NOT_FOUND

    # ------------------------------------------------------------------------------
    def getDevice(self, board):
        """ Scans connected USB devices until it finds a Pinguino board """
        logging.info("Looking for a Pinguino device ...")
        busses = usb.busses()
        for bus in busses:
            for device in bus.devices:
                logging.info("Found device 0x%04X:0x%04X" % (device.idVendor, device.idProduct))
                if (device.idVendor, device.idProduct) == (board.vendor, board.product):
                    """
                    self.configuration = device.configurations[0]
                    logging.info("Configuration = {}".format(self.configuration))
                    self.interface = self.configuration.interfaces[0][0]
                    logging.info("Interface = {}".format(self.interface))
                    self.endpoints = []
                    self.pipes = []
                    for ep in self.interface.endpoints:
                        self.endpoints.append(ep)
                        self.pipes.append(ep.address)
                    """
                    if board.bldr == "boot2":
                        self.ACTIVE_CONFIG = self.VSC_ACTIVE_CONFIG

                    return device
        return self.ERR_DEVICE_NOT_FOUND



# # ----------------------------------------------------------------------
    # def initDevice(self):
        # """ Init a Pinguino device """
        # handle = self.device.open()
        # if handle:
            # try:
                # handle.detachKernelDriver(0)
            # except:
                # pass
            # try:
                # handle.setConfiguration(self.ACTIVE_CONFIG)
            # except:
                # pass
            # try:
                # handle.claimInterface(self.INTERFACE_ID)
            # except:
                # pass
            # return handle
        # return self.ERR_USB_INIT1


    # ----------------------------------------------------------------------
    def initDevice(self, device):
        """ Init a Pinguino device """

        handle = device.open()

        if handle:

            logging.info("OS is %s" % os.getenv("PINGUINO_OS_NAME"))

            if os.getenv("PINGUINO_OS_NAME") == "linux":
                if device.idProduct == 0x003C: #self.P32_ID:
                    # make sure the hid kernel driver is not active
                    # functionality not available on Darwin or Windows
                    handle.detachKernelDriver(self.INTERFACE_ID)
                    logging.info("Kernel driver detached")


            #handle.setConfiguration(self.configuration)
            handle.setConfiguration(self.ACTIVE_CONFIG)
            logging.info("Configuration set")

            #handle.claimInterface(self.interface)
            handle.claimInterface(self.INTERFACE_ID)
            logging.info("Interface claimed")

            logging.info("Everything OK so far")
            return handle

        return self.ERR_USB_INIT1


# # ------------------------------------------------------------------------------
    # def closeDevice(self):
        # """ Close currently-open USB device """
        # try:
            # self.handle.releaseInterface()
        # except:
            # pass


    # ------------------------------------------------------------------------------
    def closeDevice(self, handle):
        """ Close currently-open USB device """
        try:
            handle.releaseInterface()
        except Exception as e:
            logging.info(e)
            pass
        logging.info("Device closed")



########################################################################
class Uploader(object):
    """Universal uploader class"""

    #----------------------------------------------------------------------
    def configure_uploader(self, hex_file, board):

        #debugger.Debugger(sys)

        self.hex_file = hex_file
        # self.filename = hex_file
        self.board = board

        if board.bldr == "noboot":

            # TODO : interface here something like PICpgm (http://www.members.aon.at/electronics/pic/picpgm/)
            #self.logwindow("You choose a board without bootloader.\nYou should either change your board type\nor use a programmer to upload your application on your board", 1)
            raise Exception("You choose a board without bootloader.\nYou should either change your board type\nor use a programmer to upload your application on your board")

        # elif board.bldr == "boot2":
            # from .uploaderVSC import uploaderVSC as Uploader

        #elif board.bldr == 'boot3':
        #    self.uploader = self.uploaderDLN(*parameters)

        elif board.arch == 8:
            from .uploader8 import uploader8 as Uploader

        elif board.arch == 32:
            from .uploader32 import uploader32 as Uploader

        self.uploader = Uploader()


    #----------------------------------------------------------------------
    # @Debugger.debug_method
    def upload_hex(self):
        self.uploader.report = []
        # self.uploader.writeHex()
        self.uploader.uploadDevice(self.hex_file, self.board)
        return self.uploader.report




#!/usr/bin/env python
#-*- coding: utf-8 -*-

from PySide import QtGui, QtCore

from .blocks import Blocks, OPENHANDCURSOR, CLOSEHANDCURSOR
from ..bloques import BlockLinear, BlockFunction, BlockNested, \
     BlockSpace, BlockNestedSecond, BlockSpaceBool, BlockFrameEdit
from ...ide.methods.decorators import Decorator


########################################################################
class ToolArea(QtGui.QWidget):

    def __init__(self, parent=None, workArea=None, ide=None):

        super(ToolArea, self).__init__(parent)

        self.ide = ide
        self.main = ide.main

        self.piecePixmaps = []
        self.pieceRects = []
        self.pieceLocations = []
        self.highlightedRect = QtCore.QRect()
        self.inPlace = 0

        self.gridLayout_2 = QtGui.QGridLayout(parent)
        self.gridLayout_2.setContentsMargins(*[0]*4)
        self.gridLayout_2.setSpacing(0)
        self.gridLayout_2.addWidget(self, 0, 0, 1, 1)

        self.isDragging = False
        self.CHILD = None
        self.POS = QtCore.QPoint(0, 0)

        self.NUEVO = False
        self.workArea = workArea

        self.setAcceptDrops(True)
        self.widget = parent


    #----------------------------------------------------------------------
    def mouse_move_event(self, event):


        onBlock, pos = self.get_child_drag()

        if not self.NUEVO and onBlock != None:

            while onBlock.objectName() != "Bloq!":
                onBlock = onBlock.parent()


            name = onBlock.NAME
            args = onBlock.ARGS
            baseName = onBlock.BASENAME
            #color = onBlock.COLOR

            finalBlock, self.npos = self.workArea().new_bloq(name, args, self.mapToGlobal(self.workArea().pos()), baseName)

            tempBlock = QtGui.QWidget(self.widget_parent)
            eval(Blocks[name])(tempBlock, args[2:])


            #print "*" * 89

            fix_pos = onBlock.pos()
            fix_pos -= QtCore.QPoint(self.scroll_area.horizontalScrollBar().value(), self.scroll_area.verticalScrollBar().value())


            tempBlock.move(fix_pos)
            tempBlock.NAME = name
            tempBlock.show()
            tempBlock.raise_()

            #tempBlock.COLOR = color

            self.finalBlock = finalBlock
            self.tempBlock = tempBlock
            self.NUEVO = True

        if onBlock != None:

            point = event.pos() - pos
            self.GlobalPoint = point+self.npos-QtCore.QPoint(10, 10)
            point -= QtCore.QPoint(self.scroll_area.horizontalScrollBar().value(), self.scroll_area.verticalScrollBar().value())

            self.tempBlock.move(point)

            self.workArea().SelectArea.hide()
            self.workArea().SelectArea.resize(QtCore.QSize(0, 0))
            self.workArea().mouseMoveEvent(None, self.GlobalPoint, self.finalBlock)
            self.workArea().expand_size(self.finalBlock)


    #----------------------------------------------------------------------
    def get_tab(self):

        return self.ide.get_tab()

    #----------------------------------------------------------------------
    @Decorator.requiere_open_files()
    def mouse_press_event(self, event):

        if not self.workArea():
            return

        self.workArea().forExpand = True
        child = self.widget_parent.childAt(event.pos())
        if child:
            while child.objectName() != "Bloq!":
                child = child.parent()
                if not child: return

            if child.childAt(child.mapFromGlobal(event.globalPos())):
                child.childAt(child.mapFromGlobal(event.globalPos())).setCursor(CLOSEHANDCURSOR)
                self.block_close_hand = child.childAt(child.mapFromGlobal(event.globalPos()))

            self.set_child_drag(child, event.pos()-child.pos())

    #----------------------------------------------------------------------
    @Decorator.requiere_open_files()
    def mouse_release_event(self, event):

        if not self.workArea():
            return

        self.workArea().forExpand = False
        child = self.get_child_drag()[0]

        try:
            self.block_close_hand.setCursor(OPENHANDCURSOR)
            del self.block_close_hand
        except: pass


        if child != None:

            if child.childAt(child.mapFromGlobal(event.globalPos())):
                child.childAt(child.mapFromGlobal(event.globalPos())).setCursor(OPENHANDCURSOR)


            while child.objectName() != "Bloq!":
                child = child.parent()

            try: self.tempBlock.deleteLater()
            except: pass

            try:
                if 0 < self.GlobalPoint.x() - self.workArea().scroll.horizontalScrollBar().value() < self.workArea().scroll.size().width() and \
                   0 < self.GlobalPoint.y() - self.workArea().scroll.verticalScrollBar().value() < self.workArea().scroll.size().height():
                    pass
                else:
                    self.finalBlock.deleteLater()
            except:
                return


            if self.finalBlock.childAt(self.finalBlock.mapFromGlobal(event.globalPos())):
                self.finalBlock.childAt(self.finalBlock.mapFromGlobal(event.globalPos())).setCursor(OPENHANDCURSOR)


            self.CHILD = None
            self.NUEVO = False


        self.main.actionSave_file.setEnabled(True)
        self.workArea().mouseReleaseEvent(event)



    #----------------------------------------------------------------------
    def set_child_drag(self, child, pos):

        self.CHILD = child
        self.POS = pos

    #----------------------------------------------------------------------
    def get_child_drag(self):

        return self.CHILD, self.POS



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/yeison/Documentos/Desarrollo/Pinguino/GitHub/pinguino-ide/pinguino/qtgui/frames/about.ui'
#
# Created: Tue Nov 17 17:51:46 2015
#      by: pyside-uic 0.2.15 running on PySide 1.2.4
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_About(object):
    def setupUi(self, About):
        About.setObjectName("About")
        About.resize(656, 507)
        self.gridLayout_2 = QtGui.QGridLayout(About)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.stackedWidget = QtGui.QStackedWidget(About)
        self.stackedWidget.setObjectName("stackedWidget")
        self.page = QtGui.QWidget()
        self.page.setObjectName("page")
        self.gridLayout = QtGui.QGridLayout(self.page)
        self.gridLayout.setObjectName("gridLayout")
        self.label_3 = QtGui.QLabel(self.page)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 2, 0, 1, 3)
        spacerItem = QtGui.QSpacerItem(620, 20, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Fixed)
        self.gridLayout.addItem(spacerItem, 5, 0, 1, 3)
        spacerItem1 = QtGui.QSpacerItem(620, 20, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Fixed)
        self.gridLayout.addItem(spacerItem1, 3, 0, 1, 3)
        self.gridLayout_4 = QtGui.QGridLayout()
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.pushButton_credits = QtGui.QPushButton(self.page)
        self.pushButton_credits.setDefault(True)
        self.pushButton_credits.setObjectName("pushButton_credits")
        self.gridLayout_4.addWidget(self.pushButton_credits, 0, 0, 1, 1)
        self.pushButton_close = QtGui.QPushButton(self.page)
        self.pushButton_close.setObjectName("pushButton_close")
        self.gridLayout_4.addWidget(self.pushButton_close, 0, 1, 1, 1)
        self.gridLayout.addLayout(self.gridLayout_4, 6, 0, 1, 3)
        self.label = QtGui.QLabel(self.page)
        self.label.setMinimumSize(QtCore.QSize(256, 0))
        self.label.setMaximumSize(QtCore.QSize(256, 16777215))
        self.label.setPixmap(QtGui.QPixmap(":/logo/art/pinguino11.png"))
        self.label.setAlignment(QtCore.Qt.AlignCenter)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 1, 1, 1, 1)
        self.label_10 = QtGui.QLabel(self.page)
        self.label_10.setText("<html><head/><body><p><a href=\"http://pinguino.cc/\"><span style=\" text-decoration: underline; color:#6299e1;\">http://pinguino.cc/</span></a></p></body></html>")
        self.label_10.setAlignment(QtCore.Qt.AlignCenter)
        self.label_10.setOpenExternalLinks(True)
        self.label_10.setObjectName("label_10")
        self.gridLayout.addWidget(self.label_10, 4, 0, 1, 3)
        spacerItem2 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem2, 1, 2, 1, 1)
        spacerItem3 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem3, 1, 0, 1, 1)
        self.label_name = QtGui.QLabel(self.page)
        font = QtGui.QFont()
        font.setPointSize(23)
        font.setWeight(75)
        font.setBold(True)
        self.label_name.setFont(font)
        self.label_name.setText("Pinguino 11.0")
        self.label_name.setAlignment(QtCore.Qt.AlignCenter)
        self.label_name.setObjectName("label_name")
        self.gridLayout.addWidget(self.label_name, 0, 0, 1, 3)
        self.stackedWidget.addWidget(self.page)
        self.page_2 = QtGui.QWidget()
        self.page_2.setObjectName("page_2")
        self.gridLayout_7 = QtGui.QGridLayout(self.page_2)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.label_9 = QtGui.QLabel(self.page_2)
        font = QtGui.QFont()
        font.setPointSize(23)
        self.label_9.setFont(font)
        self.label_9.setAlignment(QtCore.Qt.AlignCenter)
        self.label_9.setObjectName("label_9")
        self.gridLayout_7.addWidget(self.label_9, 0, 0, 1, 1)
        self.tabWidget = QtGui.QTabWidget(self.page_2)
        self.tabWidget.setObjectName("tabWidget")
        self.tab = QtGui.QWidget()
        self.tab.setObjectName("tab")
        self.gridLayout_8 = QtGui.QGridLayout(self.tab)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.label_4 = QtGui.QLabel(self.tab)
        self.label_4.setText("<html><head/><body><p align=\"center\">Jean-Pierre Mandon</p><p align=\"center\">Régis Blanchot</p><p align=\"center\">Marcus Fazzi</p><p align=\"center\">Jesus Carmona Esteban</p><p align=\"center\">Alfred Broda</p><p align=\"center\">Yeison Cardona</p><p align=\"center\">Henk Van Beek</p><p align=\"center\">Björn Pfeiffer</p><p align=\"center\">Alexis Sánchez</p></body></html>")
        self.label_4.setObjectName("label_4")
        self.gridLayout_8.addWidget(self.label_4, 0, 0, 1, 1)
        self.tabWidget.addTab(self.tab, "")
        self.tab_2 = QtGui.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.gridLayout_9 = QtGui.QGridLayout(self.tab_2)
        self.gridLayout_9.setObjectName("gridLayout_9")
        self.label_5 = QtGui.QLabel(self.tab_2)
        self.label_5.setText("<html><head/><body><p align=\"center\">Benoit Espinola</p><p align=\"center\">Sebastien Koechlin</p><p align=\"center\">Ivan Ricondo</p><p align=\"center\">Jesus Carmona Esteban</p><p align=\"center\">Marcus Fazzi</p><p align=\"center\">Régis Blanchot</p></body></html>")
        self.label_5.setObjectName("label_5")
        self.gridLayout_9.addWidget(self.label_5, 0, 0, 1, 1)
        self.tabWidget.addTab(self.tab_2, "")
        self.tab_3 = QtGui.QWidget()
        self.tab_3.setObjectName("tab_3")
        self.gridLayout_10 = QtGui.QGridLayout(self.tab_3)
        self.gridLayout_10.setObjectName("gridLayout_10")
        self.label_6 = QtGui.QLabel(self.tab_3)
        self.label_6.setText("<html><head/><body><p align=\"center\">Joan Espinoza</p><p align=\"center\">Alexis Sánchez</p><p align=\"center\">Régis Blanchot</p><p align=\"center\">Moreno Manzini </p><p align=\"center\">Yeison Cardona</p><p align=\"center\">&quot;Avrin&quot;<br/></p></body></html>")
        self.label_6.setObjectName("label_6")
        self.gridLayout_10.addWidget(self.label_6, 0, 0, 1, 1)
        self.tabWidget.addTab(self.tab_3, "")
        self.tab_4 = QtGui.QWidget()
        self.tab_4.setObjectName("tab_4")
        self.gridLayout_11 = QtGui.QGridLayout(self.tab_4)
        self.gridLayout_11.setObjectName("gridLayout_11")
        self.label_7 = QtGui.QLabel(self.tab_4)
        self.label_7.setText("<html><head/><body><p align=\"center\">France Cadet</p><p align=\"center\">Laurent Costes</p><p align=\"center\">Daniel Rodrí­guez</p></body></html>")
        self.label_7.setObjectName("label_7")
        self.gridLayout_11.addWidget(self.label_7, 0, 0, 1, 1)
        self.tabWidget.addTab(self.tab_4, "")
        self.gridLayout_7.addWidget(self.tabWidget, 1, 0, 1, 1)
        self.gridLayout_6 = QtGui.QGridLayout()
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.pushButton_license = QtGui.QPushButton(self.page_2)
        self.pushButton_license.setDefault(True)
        self.pushButton_license.setObjectName("pushButton_license")
        self.gridLayout_6.addWidget(self.pushButton_license, 0, 0, 1, 1)
        self.pushButton_close_2 = QtGui.QPushButton(self.page_2)
        self.pushButton_close_2.setObjectName("pushButton_close_2")
        self.gridLayout_6.addWidget(self.pushButton_close_2, 0, 1, 1, 1)
        self.gridLayout_7.addLayout(self.gridLayout_6, 2, 0, 1, 1)
        self.stackedWidget.addWidget(self.page_2)
        self.page_3 = QtGui.QWidget()
        self.page_3.setObjectName("page_3")
        self.gridLayout_13 = QtGui.QGridLayout(self.page_3)
        self.gridLayout_13.setObjectName("gridLayout_13")
        self.gridLayout_12 = QtGui.QGridLayout()
        self.gridLayout_12.setObjectName("gridLayout_12")
        self.pushButton_close_3 = QtGui.QPushButton(self.page_3)
        self.pushButton_close_3.setObjectName("pushButton_close_3")
        self.gridLayout_12.addWidget(self.pushButton_close_3, 0, 1, 1, 1)
        self.pushButton_about = QtGui.QPushButton(self.page_3)
        self.pushButton_about.setDefault(True)
        self.pushButton_about.setObjectName("pushButton_about")
        self.gridLayout_12.addWidget(self.pushButton_about, 0, 0, 1, 1)
        self.gridLayout_13.addLayout(self.gridLayout_12, 4, 0, 1, 1)
        spacerItem4 = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout_13.addItem(spacerItem4, 1, 0, 1, 1)
        self.label_8 = QtGui.QLabel(self.page_3)
        font = QtGui.QFont()
        font.setPointSize(23)
        self.label_8.setFont(font)
        self.label_8.setAlignment(QtCore.Qt.AlignCenter)
        self.label_8.setObjectName("label_8")
        self.gridLayout_13.addWidget(self.label_8, 0, 0, 1, 1)
        self.plainTextEdit = QtGui.QPlainTextEdit(self.page_3)
        self.plainTextEdit.setStyleSheet("background-color: rgba(255, 255, 255,0);")
        self.plainTextEdit.setFrameShape(QtGui.QFrame.NoFrame)
        self.plainTextEdit.setReadOnly(True)
        self.plainTextEdit.setPlainText("Pinguino is free software; you can redistribute it and/or modify it\n"
"under the terms of the GNU General Public License as published by the Free Software Foundation;\n"
"either version 2 of the License, or (at your option) any later version.\n"
"\n"
"Pinguino is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;\n"
"without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"
"See the GNU General Public License for more details. You should have received a copy of\n"
"the GNU General Public License along with File Hunter; if not, write to\n"
"the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA")
        self.plainTextEdit.setObjectName("plainTextEdit")
        self.gridLayout_13.addWidget(self.plainTextEdit, 2, 0, 1, 1)
        spacerItem5 = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout_13.addItem(spacerItem5, 3, 0, 1, 1)
        self.stackedWidget.addWidget(self.page_3)
        self.gridLayout_2.addWidget(self.stackedWidget, 0, 0, 1, 1)

        self.retranslateUi(About)
        self.stackedWidget.setCurrentIndex(1)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(About)

    def retranslateUi(self, About):
        About.setWindowTitle(QtGui.QApplication.translate("About", "About...", None, QtGui.QApplication.UnicodeUTF8))
        self.label_3.setText(QtGui.QApplication.translate("About", "<html><head/><body><p align=\"center\">Pinguino is an Open Software and Open Hardware<br>Arduino-like project. Boards are based on 8 or 32-bit USB built-in<br>Microchip microcontrollers. The main goal is to build a real<br>USB system without USB to serial converter.</p></body></html>", None, QtGui.QApplication.UnicodeUTF8))
        self.pushButton_credits.setText(QtGui.QApplication.translate("About", "Credits", None, QtGui.QApplication.UnicodeUTF8))
        self.pushButton_close.setText(QtGui.QApplication.translate("About", "Close", None, QtGui.QApplication.UnicodeUTF8))
        self.label_9.setText(QtGui.QApplication.translate("About", "Credits", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), QtGui.QApplication.translate("About", "Write by", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_2), QtGui.QApplication.translate("About", "Docummented by", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_3), QtGui.QApplication.translate("About", "Translated by", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_4), QtGui.QApplication.translate("About", "Art by", None, QtGui.QApplication.UnicodeUTF8))
        self.pushButton_license.setText(QtGui.QApplication.translate("About", "License", None, QtGui.QApplication.UnicodeUTF8))
        self.pushButton_close_2.setText(QtGui.QApplication.translate("About", "Close", None, QtGui.QApplication.UnicodeUTF8))
        self.pushButton_close_3.setText(QtGui.QApplication.translate("About", "Close", None, QtGui.QApplication.UnicodeUTF8))
        self.pushButton_about.setText(QtGui.QApplication.translate("About", "About", None, QtGui.QApplication.UnicodeUTF8))
        self.label_8.setText(QtGui.QApplication.translate("About", "License", None, QtGui.QApplication.UnicodeUTF8))

import resources_rc
